#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "IsoFileClasses.hpp"
#include "ClassContainer.hpp"
#include "IsoBoxTypes.hpp"


using namespace ISOFile_Include_IsoFileClasses_hpp;
using namespace ISOFile_Include_ClassContainer_hpp;
using namespace ISOFile_Include_IsoBoxTypes_hpp;

#define BAD_FRAME_THRESHOLD 2.0

typedef char fixstring[256];

typedef struct
{  
  fixstring orig_filename;        // The original YUV-Sequence
  fixstring rec_filename;         // The reconstructed YUV-Sequence with quantazation parameter QP1
  fixstring dec_filename;         // The reconstructed YUV-Sequence with quantazation parameter QP1
} InputParameters;

typedef struct image 
{
  int timestamp;             
  int width; 
  int height;          /* Picture dimension Y */
  unsigned char		*buf;
  int uwidth; 
  int uheight;          /* Picture dimension U*/
  unsigned char		*ubuf;
  int vwidth; 
  int vheight;          /* Picture dimension V*/
  unsigned char		*vbuf;
} IMAGE;

typedef struct  
{
 char   type[100];
 int		width; 
 int    height;          /* Picture dimension */
 int    length;
 int    vfr;
 float  framerate;
 int    filesize;
 int    timeindex;       /* counter in the video file */
 int    frameindex;
 ClassContainer headerContainer;
 BufferManager fileDecBufferManager;
 FILE   *fp;
} SA4VideoFile;

#define FILE_MODE_SA4 0
#define FILE_MODE_ISO 1

void ImgAllocBuf( IMAGE *img, SA4VideoFile *fd)
{
  int	size;
  img->width = fd->width;
  img->height = fd->height;
	size = img->width * img->height;
	
	if ( !( img->buf = (unsigned char *)malloc(size) ) )
	{
	  printf("memory allocation error!\n");
	  exit(-1);
	}

	img->uwidth=img->width/2;
	img->uheight=img->height/2;
	size = img->uwidth * img->uheight;
	if ( !( img->vbuf = (unsigned char *)malloc(size) ) )
	{
	  printf("memory allocation error!\n");
	  exit(-1);
	}
	
  img->vwidth=img->width/2;
	img->vheight=img->height/2;
	size = img->vwidth * img->vheight;
	if ( !( img->ubuf = (unsigned char *)malloc(size) ) )
	{
	  printf("memory allocation error!\n");
	  exit(-1);
	}
	
}

SA4VideoFile *ImgOpenForReading( char *file_name )
{
  char line[1000];
  SA4VideoFile 	*fd = new SA4VideoFile[1];
	
	if ( !( fd->fp = fopen( file_name, "rb" ) ) )
	{
	  printf( "Couldn't open input file --> %s\n", file_name );
	  exit(-1);
	}
	
	fgets(line, 1000, fd->fp); 			
	fscanf(fd->fp, "%*s %*s %d\n", &fd->width); 			
	fscanf(fd->fp, "%*s %*s %d\n", &fd->height); 			
	fscanf(fd->fp, "%*s %*s %d\n", &fd->filesize); 
    fscanf(fd->fp, "%*s %*s %d\n", &fd->vfr);				
	fscanf(fd->fp, "%*s %*s %f\n", &fd->framerate);	/* framerate 0 means that 90 kHz timestamps are assumed */
	fscanf(fd->fp, "%*s %*s %d\n", &fd->length);

    fd->timeindex = -1;
    fd->frameindex = 0;
  
	return fd;
}

SA4VideoFile *IsoFileOpenForReading( char *file_name )
{
    SA4VideoFile 	*fd = new SA4VideoFile[1];
	
	if ( !( fd->fp = fopen( file_name, "rb" ) ) )
	{
	  printf( "Couldn't open input file --> %s\n", file_name );
	  exit(-1);
	}
        const int rdSz = 16*1024;
        unsigned char decBuffer[rdSz];
        int ret;
        
        
	    ret = fread(decBuffer, 1, rdSz,fd->fp);
        if(ret < rdSz)
        {
            printf("Could not read ISO header!\n");
		    exit(-1);
        }
        
        ClassContainer *myDecClassContainer = &fd->headerContainer;
        BufferManager *fileDecBufferManager = &fd->fileDecBufferManager;
        
        fileDecBufferManager->writeData((Byte *)decBuffer,rdSz);
        
        try
        {
            myDecClassContainer->decodeAll(fileDecBufferManager);
        }
        catch(int)
        {
           ;
        }
        catch(char *str)
        {
           printf("Decode ISO Header, exception raised: %s\n",str);
        }

        SampleDescriptionBox *sdb = (SampleDescriptionBox *)myDecClassContainer->getBoxRecursive(IsoBoxTypes::type_SampleDescriptionBox);
        MediaHeaderBox *mdhd = (MediaHeaderBox *)myDecClassContainer->getBoxRecursive(IsoBoxTypes::type_MediaHeaderBox);
        TimeToSampleBox *tts = (TimeToSampleBox *)myDecClassContainer->getBoxRecursive(IsoBoxTypes::type_TimeToSampleBox);
        SampleSizeBox *sampSz = (SampleSizeBox *)myDecClassContainer->getBoxRecursive(IsoBoxTypes::type_SampleSizeBox);
        ChunkOffsetBox *chOff = (ChunkOffsetBox *)myDecClassContainer->getBoxRecursive(IsoBoxTypes::type_ChunkOffsetBox);

        if(sdb==NULL || mdhd==NULL || tts==NULL || sampSz==NULL || chOff==NULL)
        {
            printf("Could not find required headers from ISO file %s, Exiting!\n",file_name);
		    exit(-1);
        }

        VisualSampleEntry *vse = (VisualSampleEntry *)sdb->sampleEntry[0];
        
        tts->decode_sample_delta();

        fd->width = vse->width;
        fd->height = vse->height;
        fd->filesize = 0; 
        fd->vfr = tts->min_sample_delta != tts->max_sample_delta;
        fd->framerate = (float)mdhd->timescale/(float)tts->min_sample_delta;

    fd->timeindex = -1;
    fd->frameindex = 0;
  
	return fd;
}

int ImgRead( char *file_name, IMAGE *img, SA4VideoFile *fd, int fileMode )
{
    int	size, rsize;    

    if(!fd->vfr)
      fd->timeindex++;
    else if(fileMode == FILE_MODE_SA4)
    {
      if( fread(&fd->timeindex, 1, sizeof(int), fd->fp) < sizeof(int) ) 
          return 0;
    }     
    else
    {
        if(fd->frameindex == 0)
            fd->timeindex = 0;
        else
        {
            TimeToSampleBox *tts = (TimeToSampleBox *)fd->headerContainer.getBoxRecursive(IsoBoxTypes::type_TimeToSampleBox);
            fd->timeindex += (tts->decoded_sample_delta[fd->frameindex]/tts->min_sample_delta);
        }
    }

    fd->frameindex++;
    
    /* Adapt Framerate to 90 kHz */
    if( fd->framerate > 0. )
      img->timestamp = (int)((float)(90000*fd->timeindex)/(float)(fd->framerate)+0.5);
        
	  size = img->width * img->height;
	  if ( (rsize=(int)fread(img->buf, sizeof(unsigned char), size, fd->fp)) < size ) 
	     return 0;

	  size = img->uwidth * img->uheight;
	  if ( (rsize=(int)fread(img->ubuf, sizeof(unsigned char), size, fd->fp)) < size ) 
	     return 0;

	  size = img->vwidth * img->vheight;
	  if ( (rsize=(int)fread(img->vbuf, sizeof(unsigned char), size, fd->fp)) < size ) 
	     return 0;
    
    return 1;
}

int UnreadImg( char *file_name, IMAGE *img, SA4VideoFile *fd, int fileMode)
{
    long int	size;
    
    size = sizeof(unsigned char)*(img->width * img->height + img->uwidth * img->uheight + img->vwidth * img->vheight);
        
    if(fd->vfr && fileMode == FILE_MODE_SA4)
       size += sizeof(int);

    fd->frameindex -= 2;

    TimeToSampleBox *tts = (TimeToSampleBox *)fd->headerContainer.getBoxRecursive(IsoBoxTypes::type_TimeToSampleBox);

    for(int index = fd->frameindex ; index <= fd->frameindex+1 ; index++)
    {
        fd->timeindex -= (tts->decoded_sample_delta[index]/tts->min_sample_delta);
    }
    
    /* go back two times and read old image */
    if(fseek(fd->fp, -2*size, SEEK_CUR) != EOF)
      return ImgRead(file_name, img, fd,fileMode);
    else
      return EOF;     
}

/* PSNR of Y component */
double quality(IMAGE *orig, IMAGE *rec)
{
  unsigned int i;
  int diff;
  unsigned dimension = orig->width*orig->height;
  double error=0;
  for ( i=0; i<dimension; i++ ) 
  {
    diff = orig->buf[i]-rec->buf[i];
    error += (double) (diff*diff);
  }
  error /= (double) dimension;  
  return error; /* psnr in dB */
}

double psnr(double error)
{
       return (double)(10.0 * log10 (255.0*255.0/error));
}

int main( int argc, char *argv[] )
{
  int              NumberOfOriginalFrames, NumberOfBadFrames, NumberOfBadFrames2, NumberOfBadFramesContinuous;
  int              NumberOfEncodedFrames, NumberOfDecodedFrames, NumberOfSkippedFrames, NumberOfSkippedFrames2, NumberOfLostFrames, NumberOfLostFrames2;
  double           recerror, recerrorsum, decerror, decerrorsum;
  double           recpsnr, recpsnrsum, decpsnr, decpsnrsum;
  double           rec2psnrsum, dec2psnrsum;
  double           decFramerateSum,decFramerateSum2;
  IMAGE            org, rec, dec; 
  SA4VideoFile     *fd1, *fd2, *fd3;
  FILE             *fd4;
  InputParameters  inps, *inp=&inps;
  int              con_skipped_frames=0, con_lost_frames=0;
  int              tmp_con_skipped_frames=0, tmp_con_lost_frames=0;
  int              playout_delay = 0; /* Playout Delay at receiver - some jitter might be allowed */
  int              lastDecodedTs = 0;
  int              firstDecodedFrame = 1;
  double           AEP,PEANSD,ADP,PDANSD,SDEP,SDDP,PDVD,SDPDVD,STIDR;
  int              fileMode;

  if (argc != 4)
  {
    printf( "\nUsage: QualEval [Original File] [Reconstructed File] [Decoded File]\n" );
    return(0);
  }

  strcpy(inp->orig_filename,argv[1]);
  strcpy(inp->rec_filename,argv[2]);
  strcpy(inp->dec_filename,argv[3]);

  if(!stricmp(&inp->rec_filename[strlen(inp->rec_filename)-4], ".sa4"))
    fileMode = FILE_MODE_SA4;
  else if(!stricmp(&inp->rec_filename[strlen(inp->rec_filename)-4], ".iso"))
    fileMode = FILE_MODE_ISO;
  else
  {
	  printf( "Unknown file extension!\n");
	  exit(-1);
  }

  /* File Opening */
  fd1=ImgOpenForReading(inp->orig_filename);
  
  if(fileMode == FILE_MODE_SA4)
  {
      fd2=ImgOpenForReading(inp->rec_filename);
      fd3=ImgOpenForReading(inp->dec_filename);
  }
  else
  {
      fd2=IsoFileOpenForReading(inp->rec_filename);
      fd3=IsoFileOpenForReading(inp->dec_filename);
  }

  /* Memory Allocation */
  ImgAllocBuf(&org, fd1);
  ImgAllocBuf(&rec, fd2);
  ImgAllocBuf(&dec, fd3);
  

  if ( !( fd4 = fopen( "Results\\details.txt", "w" ) ) )
	{
	  printf( "Couldn't open output file --> %s\n", "details.txt" );
	  exit(-1);
	}
  
  /* Initialization */
  recerrorsum =0.;
  decerrorsum =0.;
  recpsnrsum = 0.;
  decpsnrsum = 0.;
  rec2psnrsum = 0.;
  dec2psnrsum = 0.;
  NumberOfOriginalFrames = 0;
  NumberOfEncodedFrames = 0;
  NumberOfDecodedFrames = 0;
  NumberOfBadFrames = 0;
  NumberOfBadFrames2 = 0;
  NumberOfSkippedFrames = 0;
  NumberOfSkippedFrames2 = 0;
  NumberOfLostFrames = 0;
  NumberOfLostFrames2 = 0;
  NumberOfBadFramesContinuous = 0;
  decFramerateSum = 0;
  decFramerateSum2 = 0;
  
  
  /* Read image from encoded file */
  while( ImgRead(inp->rec_filename, &rec, fd2, fileMode) )
  {
   /* Read corresponding image from original file */
   con_skipped_frames = 0;
   while( ImgRead(inp->orig_filename, &org, fd1, FILE_MODE_SA4) )
     {
     NumberOfOriginalFrames++;
     if(org.timestamp >= rec.timestamp)
     {
       NumberOfSkippedFrames += con_skipped_frames;
       NumberOfSkippedFrames2 += (con_skipped_frames*con_skipped_frames);
       tmp_con_skipped_frames = con_skipped_frames;
       con_skipped_frames = 0;
       break;
     }
     else
     {
       con_skipped_frames++;
     }
     }   
   
   /* Now I should have two matching frames */ 

   /* Metric Evaluation     */
   /* Average Encoding PSNR */
   recerror = quality(&org, &rec);
   recerrorsum += recerror; 
   recpsnr = psnr(recerror);
   recpsnrsum += recpsnr;
   
   /* Standard Deviation of Encoding PSNR */
   rec2psnrsum += (recpsnr*recpsnr);
   

   /* Read corresponding image from decoded file */
   if(ImgRead(inp->dec_filename, &dec,  fd3, fileMode))
     {                         
       if( dec.timestamp > (rec.timestamp-playout_delay) )
       {
           UnreadImg(inp->dec_filename, &dec,  fd3, fileMode);
           con_lost_frames++;
       }
       else
       {
           NumberOfLostFrames += con_lost_frames;
           NumberOfLostFrames2 += (con_lost_frames*con_lost_frames);
           NumberOfDecodedFrames++;
           tmp_con_lost_frames = con_lost_frames;
           con_lost_frames = 0;

           if(!firstDecodedFrame)
           {
                double frameRate = 90000/(double)(dec.timestamp - lastDecodedTs);
                decFramerateSum += frameRate;
                decFramerateSum2 += (frameRate*frameRate);
           }

           lastDecodedTs = dec.timestamp;

           firstDecodedFrame = 0;
       }
     }
      
   /* Average Decoding PSNR */
   decerror = quality(&org, &dec);
   decerrorsum += decerror; 
   decpsnr = psnr(decerror);
   decpsnrsum += decpsnr;
   
   /* Standard Deviation of Decoding PSNR */
   dec2psnrsum += (decpsnr*decpsnr);
   
   /* Average Error Length */
   if( (recpsnr - decpsnr) > BAD_FRAME_THRESHOLD )
   {
       NumberOfBadFramesContinuous++;
   }
   else
   {
       NumberOfBadFrames += NumberOfBadFramesContinuous;
       NumberOfBadFrames2 += (NumberOfBadFramesContinuous*NumberOfBadFramesContinuous);
       NumberOfBadFramesContinuous = 0;
   }   

   fprintf(fd4, "%8d\t%8d\t%8.4f\t%8.4f\t%4d\t%4d\t%4d\n", 
                rec.timestamp, NumberOfEncodedFrames, recpsnr, decpsnr, 
                ( (recpsnr - decpsnr) > BAD_FRAME_THRESHOLD ) ? 1 : 0,
                 tmp_con_skipped_frames, tmp_con_lost_frames);
   
   NumberOfEncodedFrames++;
  }

  /*Average Encoding PSNR (AEP)*/
  AEP = recpsnrsum/(double)(NumberOfEncodedFrames);
  /*PSNR of Encoding Average Normalized Square Difference (PEANSD)*/
  PEANSD = psnr(recerrorsum/(double)(NumberOfEncodedFrames));
  /*Average Decoding PSNR (ADP)*/
  ADP = decpsnrsum/(double)(NumberOfEncodedFrames);
  /*PSNR of Decoding Average Normalized Square Difference (PDANSD)*/
  PDANSD = psnr(decerrorsum/(double)(NumberOfEncodedFrames));
  /*Standard Deviation of Encoding PSNR (SDEP)*/
  SDEP = sqrt( rec2psnrsum/(double)(NumberOfEncodedFrames) - pow(recpsnrsum/(double)(NumberOfEncodedFrames), 2 ) );
  /*Standard Deviation of Decoding PSNR (SDDP)*/
  SDDP = sqrt( dec2psnrsum/(double)(NumberOfEncodedFrames) - pow(decpsnrsum/(double)(NumberOfEncodedFrames), 2 ) );
  /*Percentage Degraded Video Duration (PDVD)*/
  PDVD = (double)(NumberOfBadFrames)/(double)(NumberOfEncodedFrames)*100;
  /*Standard Deviation of Percentage Degraded Video Duration (SDPDVD)*/
  SDPDVD = sqrt( NumberOfBadFrames2/(double)(NumberOfEncodedFrames) - pow(NumberOfBadFrames/(double)(NumberOfEncodedFrames), 2 ) ); 
  /*Standard Deviation of Instantaneous Frame Rate (STIDR)*/
  STIDR = sqrt( decFramerateSum2/(double)(NumberOfDecodedFrames-1) - pow(decFramerateSum/(double)(NumberOfDecodedFrames-1), 2 ) ); 

  printf("%4d\t%4d\t%4d\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\t%6.4lf\n", 
          NumberOfOriginalFrames,
          NumberOfEncodedFrames, 
          NumberOfDecodedFrames,
          AEP,
          PEANSD,
          ADP,
          PDANSD,
          SDEP,
          SDDP,
          PDVD,
          SDPDVD,
          (double)(NumberOfSkippedFrames)/(double)(NumberOfOriginalFrames)*100,
          sqrt( NumberOfSkippedFrames2/(double)(NumberOfOriginalFrames) - pow(NumberOfSkippedFrames/(double)(NumberOfOriginalFrames), 2 ) ),
          STIDR); 

  fclose(fd1->fp);
  fclose(fd2->fp);
  fclose(fd3->fp);
  
  return 0;
}    

