// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#ifndef _ISOFILE_INCLUDE_BUFFER_MANAGE_HPP_
#define _ISOFILE_INCLUDE_BUFFER_MANAGE_HPP_

#include "Namespace.hpp"
#include "Vector.hpp"

DECLARE_NAMESPACE(ISOFile_Include_BufferManager_hpp);

using namespace ISOFile_Include_DataTypes_hpp;
using namespace ISOFile_Include_Vector_hpp;

  /**
    *   Class BufferManager, provides the functionality for endian handling, reading and writing data from and to a vector buffer in "Network Byte Order"
    *   
    */
class BufferManager
{            
    public:
      /**
        *   Buffer Manager constructor
        *
        *   @param  preferredAllocSize      The allocation size initially
        *   @param  preferredAllocChunkSize The chunk size used for each re-allocation
        *
        */
        BufferManager(UInt32 preferredAllocSize = defaultAllocChunkSize, UInt32 preferredAllocChunkSize = defaultAllocChunkSize);

      /**
        *   Buffer Manager destructor
        */
        ~BufferManager();

    public:

      /**
        *   Return the actual storage area of data (the "network bytes" as a vector) along with the size of the data contained.
        *   
        *   @param  buffer  Place where the addess of the storage is saved
        *
        *   @return The data size in the storage
        */
        UInt32 getBuffer(Byte **buffer);

      /**
        *   Get the current write pointer in the buffer
        */
        UInt32 getWritePointer();

      /**
        *   Set the network write pointer to desired value.Has no other checks and effects. This will not effect the data size
        *   of the buffer until something is actually written to the new location.
        *
        *   @param  newWritePointer The new value of network write pointer to set.
        *
        *   @return The updated write address
        */
        UInt32 setWritePointer(UInt32 newWritePointer);

      /**
        *   Get the current write pointer
        *
        */
        UInt32 getReadPointer();

      /**
        *   Set the network read pointer to desired value. Has no other checks and effects. This will not effect the data size
        *   of the buffer until something is actually read from the new location.
        *
        *   @param  newReadPointer The new value of network read pointer to set.
        *
        *   @return The updated read address
        */
        UInt32 setReadPointer(UInt32 newReadPointer);

      /**
        *   Add directly bytes of data provided into storage. The bytes are assumed to be in network order
        *   
        *   @param  buffer  Address of location from where data is to be added
        *   @param  count   Number of bytes to be added
        */
        UInt32 writeData(Byte *buffer, UInt32 count);
      /**
        *   Write a byte of network byte to the buffer at the write pointer. The write pointer and data size are updated.
        *
        *   @param  data    The byte of data to write
        *
        *   @return The new write pointer
        */
        UInt32 writeByte(Byte data);

      /**
        *   Write a 16 bit unsigned short int. Endian convesion to network byte order is also done. The write pointer and data size are updated.
        *
        *   @param  data    The data to write
        *
        *   @return The new write pointer
        */
        UInt32 writeUInt16(UInt16 data);

      /**
        *   Write a 32 bit unsigned int. Endian convesion to network byte order is also done. The write pointer and data size are updated.
        *
        *   @param  data    The data to write
        *
        *   @return The new write pointer
        */
        UInt32 writeUInt32(UInt32 data);
        
      /**
        *   Write bits in multiple of 8. The write pointer and data size are updated
        *
        *   @param  bitCount    Number of bits to write, must be a multiple of 8
        *   @param  data        The bits of data to write
        *
        *   @return The new write pointer
        */
        UInt32 writebit(UInt32 bitCount, UInt32 data);
        
      /**
        *   Read directly bytes of data into storage.
        *   
        *   @param  buffer  Address of location where data is to be added
        *   @param  count   Number of bytes to read
        */
        UInt32 readData(Byte *buffer, UInt32 count);

      /**
        *   Read a byte of network byte to the buffer at the write pointer. The read pointer is updated.
        *
        *   @param  data    The location where read byte is stored
        *
        *   @return The new read pointer
        */
        UInt32 readByte(Byte *data);

      /**
        *   Read a 16 bit unsigned short int. Endian convesion from network byte order is also done. The read pointer is updated.
        *
        *   @param  data    The location where read data is stored
        *
        *   @return The new read pointer
        */
        UInt32 readUInt16(UInt16 *data);

      /**
        *   Read a 32 bit unsigned int. Endian convesion from network byte order is also done. The read pointer is updated.
        *
        *   @param  data    The location where read data is stored
        *
        *   @return The new read pointer
        */
        UInt32 readUInt32(UInt32 *data);

      /**
        *   Read bits in multiple of 8. The read pointer is updated
        *
        *   @param  bitCount    Number of bits to read, must be a multiple of 8
        *   @param  data        The location where read data is stored
        *
        *   @return The new read pointer
        */
        UInt32 readbit(UInt32 bitCount, UInt32* data);

    private:
        Vector<Byte> networkISOFileData;
        UInt32 networkReadPointer;
        UInt32 networkWritePointer;
        UInt32 maxNetworkWritePointer;
        static UInt32 defaultAllocChunkSize;
        
    private:
        UInt32 incrementReadPointer(UInt32 increment);
        
        UInt32 incrementWritePointer(UInt32 increment);

};


END_NAMESPACE(ISOFile_Include_BufferManager_hpp);

#endif  //_ISOFILE_INCLUDE_BUFFER_MANAGE_HPP_

