// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#ifndef _ISOFILE_INCLUDE_ISO_FILE_CLASSES_HPP_
#define _ISOFILE_INCLUDE_ISO_FILE_CLASSES_HPP_

#include "Namespace.hpp"
#include "ISOFileDataTypes.hpp"
#include "BufferManager.hpp"

DECLARE_NAMESPACE(ISOFile_Include_IsoFileClasses_hpp);

using namespace ISOFile_Include_IsoFileDataTypes_hpp;
using namespace ISOFile_Include_BufferManager_hpp;

/**
  *   ISO/IEC 14496-12:2005(E) Class Box, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class Box
{
    public:
        
       /**
         *   Box Constructor
         *   
         */
        Box();
       
       /**
         *   Box Constructor with known boxtype string
         *   
         */
        Box(const char *boxtype);
       
       /**
         *   Box Constructor with known boxtype
         *   
         */
        Box(unsigned_int32 boxtype);
       
       /**
         *   Box Constructor with known boxtype and extended_type
         *   
         */
        Box(unsigned_int32 boxtype, unsigned_int8 extended_type[16]);
        
       /**
         *   Box Distructor
         *   
         */
        virtual ~Box();
        
    public:

        unsigned_int32 size;
        unsigned_int32 type;
        
    public:
        
      /**
        *   Decodes a buffer encoded according to ISO/IEC 14496-12:2005(E) and puts the data in respective data members of the class.
        *   
        *   @param  buffer  The location of encoded data
        *   @param  count   The amount of data in buffer
        */
        virtual void decode(Byte *buffer, UInt32 count);
        
      /**
        *   Encodes a buffer encoded according to ISO/IEC 14496-12:2005(E) from the data stored respective data members of the class. A buffer is allocated for this purpose, freed on object destruction.
        *   
        *   @param  buffer  The location where to put the encoded data
        *   @param  count   The amount of data written in buffer
        */
        virtual void encode(Byte **buffer, UInt32 *count);
        
      /**
        *   Returns the estimated size of data in the class
        *
        */
        virtual UInt32 estimateSize();
        
      /**
        *   Returns the current write pointer
        *
        */
        UInt32 getCurrentWritePointer();

      /**
        *   Returns the current read pointer
        *
        */
        UInt32 getCurrentReadPointer();

      /**
        *   Get the buffer written so far.
        *   
        *   @param  buffer  The buffer pointer will be returned here
        *
        *   @return The current size of stored data
        */
        UInt32 getBuffer(Byte **buffer);

      /**
        *   Scans the box and contained boxes recursively for a given type. Returns the first occurance of the box encountered that matches the type.
        *   Always returns NULL for classes that cannot contain other classes according to ISO/IEC 14496-12:2005(E). Only the implementation in class ClassContainer
        *   can return an object.
        *
        *   @see    ClassContainer
        *   
        *   @param  type    The type of box to look for
        *
        *   @return Pointer of Box if found, NULL otherwise.
        */
        virtual Box* getBoxRecursive(const char* type){return NULL;}
        
      /**
        *   Converts the source bytes to a NULL terminated string.
        *   
        *   @param  src     An unsigned 32 bit buffer containing 4 characters
        *   @param  target  Location where to store the converted string
        */
        static void mapUInt32ToString(unsigned_int32 *src, char *target);

      /**
        *   Converts the source 4 characters to a packed 32 bit value.
        *   
        *   @param  src     The location with 4 characters to pack
        *   @param  target  Location to store the packed data
        */
        static void mapStringToUInt32(char *src, unsigned_int32 *target);

    protected:
        void update_size();
        void write_size();

    protected:
       BufferManager *bufManager;

    private:
		void set_type(const char *str);
        void allocBufManager();

};

/**
  *   ISO/IEC 14496-12:2005(E) Class FullBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class FullBox: public Box
{
    public:
        FullBox();
        FullBox(const char *boxtype, unsigned_int8 v, bit24 f);
        FullBox(unsigned_int32 boxtype, unsigned_int8 v, bit24 f);
        virtual ~FullBox();
       
    public:
        unsigned_int8 version;
        bit24 flags;

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        virtual void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        virtual void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        virtual UInt32 estimateSize();
};


/**
  *   ISO/IEC 14496-12:2005(E) Class FileTypeBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class FileTypeBox: public Box 
{
    public:
        FileTypeBox();
        virtual ~FileTypeBox();
        
    public:
        unsigned_int32 major_brand;
        unsigned_int32 minor_version;
        Vector<unsigned_int32> compatible_brands;

    public:

      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
       
    private:
		void init();
};

END_NAMESPACE(ISOFile_Include_IsoFileClasses_hpp);

#include "ClassContainer.hpp"

using namespace ISOFile_Include_ClassContainer_hpp;

DECLARE_NAMESPACE(ISOFile_Include_IsoFileClasses_hpp);


/**
  *   ISO/IEC 14496-12:2005(E) Class MovieBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MovieBox: public Box, public ClassContainer
{
    public:
        MovieBox();
        virtual ~MovieBox();
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
        
      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class MediaDataBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MediaDataBox: public Box
{
    public:
        MediaDataBox();
        virtual ~MediaDataBox();
        
    public:
        Vector<bit8> data;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add directly bytes of data provided into MediaDataBox.
        *   
        *   @param  dataToAdd   Address of location from where data is to be added
        *   @param  count       Number of bytes to be added
        *
        */
        void addData(bit8* dataToAdd, UInt32 count);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class FreeSpaceBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class FreeSpaceBox: public Box
{
    public:
        FreeSpaceBox();
        virtual ~FreeSpaceBox();
        
    public:
        Vector<bit8> data;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add directly bytes of data provided into FreeSpaceBox.
        *   
        *   @param  dataToAdd   Address of location from where data is to be added
        *   @param  count       Number of bytes to be added
        *
        */
        void addData(bit8* dataToAdd, UInt32 count);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class MovieHeaderBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MovieHeaderBox: public FullBox
{
    public:
        MovieHeaderBox(unsigned_int8 version);
        virtual ~MovieHeaderBox();
    
    public:
        unsigned_int32 creation_time;
        unsigned_int32 modification_time;
        unsigned_int32 timescale;
        unsigned_int32 duration;
        int32 rate;
        int16 volume;
        static const bit16 reserved;
        static const unsigned_int32 reserved_arr[2];
        int32 matrix[9];
        bit32 pre_defined[6];
        unsigned_int32 next_track_ID;

    public:
        static const UInt32 timeFrom1904To1970;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Get current time for MovieHeaderBox, TrackHeaderBox, MediaHeaderBox etc. as specified in ISO/IEC 14496-12:2005(E)
        *
        */
        static unsigned_int32 getCurrentISOFileFormatTime();
};


/**
  *   ISO/IEC 14496-12:2005(E) Class TrackBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class TrackBox: public Box, public ClassContainer
{
    public:
        TrackBox();
        virtual ~TrackBox();

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
        
      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class TrackHeaderBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class TrackHeaderBox: public FullBox
{
    public:
        TrackHeaderBox(unsigned_int8 version, bit24 flags);
        virtual ~TrackHeaderBox();
        
    public:
        unsigned_int32 creation_time;
        unsigned_int32 modification_time;
        unsigned_int32 track_ID;
        static const unsigned_int32 reserved;
        unsigned_int32 duration;
        static const unsigned_int32 reserved_arr[2];
        int16 layer;
        int16 alternate_group;
        int16 volume;
        static const unsigned_int16 reserved_2;
        int32 matrix[9];
        unsigned_int32 width;
        unsigned_int32 height;

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set ISO/IEC 14496-12:2005(E) TrackHeaderBox::volume
        *
        */
        void set_volume(UInt32 track_is_audio);

};


/**
  *   ISO/IEC 14496-12:2005(E) Class MediaBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MediaBox: public Box, public ClassContainer 
{
    public:
        MediaBox();
        virtual ~MediaBox();

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class MediaHeaderBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MediaHeaderBox: public FullBox
{
    public:
        MediaHeaderBox(unsigned_int8 version);
        virtual ~MediaHeaderBox();
        
    public:
        unsigned_int32 creation_time;
        unsigned_int32 modification_time;
        unsigned_int32 timescale;
        unsigned_int32 duration;
        bit1 pad;
        unsigned_int5 language[3]; // ISO-639-2/T language code
        unsigned_int16 pre_defined;

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set ISO/IEC 14496-12:2005(E) setLanguageFromISOCode::language
        *
        *   @param  isoLanguageCode 3 character ISO language code in network byte order
        *
        */
        void setLanguageFromISOCode(char *isoLanguageCode);

      /**
        *   Get ISO/IEC 14496-12:2005(E) setLanguageFromISOCode::language
        *
        *   @param  isoLanguageCode 3 character ISO language code in network byte order
        *
        */
        void getLanguageFromISOCode(char *isoLanguageCode);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class HandlerBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class HandlerBox: public FullBox
{
    public:
        HandlerBox(unsigned_int8 version = 0);
        virtual ~HandlerBox();
        
    public:
        unsigned_int32 pre_defined;
        unsigned_int32 handler_type;
        unsigned_int32 reserved[3];
        string name;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);
        
      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
        
      /**
        *   Set ISO/IEC 14496-12:2005(E) HandlerBox::name
        *
        *   @param  new_name NULL terminated name string
        *
        */
        void set_name(string new_name);

    private:
        void alloc_name(UInt32 size);
        void clear_name();

};


/**
  *   ISO/IEC 14496-12:2005(E) Class MediaInformationBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class MediaInformationBox: public Box, public ClassContainer 
{
    public:
        MediaInformationBox();
        virtual ~MediaInformationBox();

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class VideoMediaHeaderBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class VideoMediaHeaderBox: public FullBox 
{
    public:
        VideoMediaHeaderBox(unsigned_int8 version = 0);
        virtual ~VideoMediaHeaderBox();

    public:
        unsigned_int16 graphicsmode;
        unsigned_int16 opcolor[3];
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
};


/**
  *   ISO/IEC 14496-12:2005(E) Class DataInformationBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class DataInformationBox: public Box, public ClassContainer 
{
    public:
        DataInformationBox();
        virtual ~DataInformationBox();

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();
        
      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class DataEntryUrlBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class DataEntryUrlBox: public FullBox
{
    public:
        DataEntryUrlBox(bit24 flags, unsigned_int8 version = 0);
        virtual ~DataEntryUrlBox();
        
    public:
        string location;
        
    public:
        UInt32 locationSize;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set ISO/IEC 14496-12:2005(E) DataEntryUrlBox::location
        *
        *   @param  new_location NULL terminated location string
        *
        */
        void set_location(string new_location);

    private:
        void alloc_location(UInt32 size);
        void clear_location();
        
};


/**
  *   ISO/IEC 14496-12:2005(E) Class DataEntryUrnBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class DataEntryUrnBox: public FullBox
{
    public:
        DataEntryUrnBox(bit24 flags, unsigned_int8 version = 0);
        virtual ~DataEntryUrnBox();
        
    public:
        string name;
        string location;
        
    public:
        UInt32 nameSize;
        UInt32 locationSize;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set ISO/IEC 14496-12:2005(E) DataEntryUrnBox::name
        *
        *   @param  new_location NULL terminated name string
        *
        */
        void set_name(string new_location);

      /**
        *   Set ISO/IEC 14496-12:2005(E) DataEntryUrnBox::location
        *
        *   @param  new_location NULL terminated location string
        *
        */
        void set_location(string new_location);

    private:
        void alloc_name(UInt32 size);
        void clear_name();
        
        void alloc_location(UInt32 size);
        void clear_location();

};


/**
  *   ISO/IEC 14496-12:2005(E) Class DataReferenceBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class DataReferenceBox: public FullBox, public ClassContainer
{
    public:
        DataReferenceBox(bit24 flags, unsigned_int8 version = 0);
        virtual ~DataReferenceBox();
        
    public:
        unsigned_int32 entry_count;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add DataEntryUrnBox/DataEntryUrlBox
        *
        */
        Box* addNewBox(const char* type);

};


/**
  *   ISO/IEC 14496-12:2005(E) Class SampleTableBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class SampleTableBox: public Box, public ClassContainer
{
    public:
        SampleTableBox();
        virtual ~SampleTableBox();

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   @see    ClassContainer::getBoxRecursive
        *
        */
        Box* getBoxRecursive(const char* type);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class TimeToSampleBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class TimeToSampleBox: public FullBox
{
    public:
        TimeToSampleBox(unsigned_int8 version = 0);
        virtual ~TimeToSampleBox();
        
    public:
        unsigned_int32 entry_count;
        Vector<unsigned_int32> sample_count;
        Vector<unsigned_int32> sample_delta;

    public:
        
      /**
        *   Decoded differentially encoded TimeToSampleBox::sample_delta in a linear array, one entry corresponding to each sample
        *
        *   @see    TimeToSampleBox::decode_sample_delta
        *
        */
        Vector<unsigned_int32> decoded_sample_delta;
      
      /**
        *   The minimum value in the decoded TimeToSampleBox::decode_sample_delta
        *
        *   @see    TimeToSampleBox::decode_sample_delta
        *
        */
        unsigned_int32 min_sample_delta;
      
      /**
        *   The max value in the decoded TimeToSampleBox::decode_sample_delta
        *
        *   @see    TimeToSampleBox::decode_sample_delta
        *
        */
        unsigned_int32 max_sample_delta;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add a new TimeToSampleBox::sample_delta
        *
        *   @return The new TimeToSampleBox::entry_count
        *
        */
        UInt32 add_sample_delta(unsigned_int32 new_sample_delta);

      /**
        *   Decode the differentially encoded TimeToSampleBox::sample_delta to linear array of decoded_sample_delta, one entry corresponding to each sample
        *
        *   @see    TimeToSampleBox::decoded_sample_delta
        *
        */
        UInt32 decode_sample_delta();
      
      /**
        *   Get sum of all sample deltas, i.e., media duration
        *
        */
        UInt32 getCumulativeSampleDelta();
};


/**
  *   ISO/IEC 14496-12:2005(E) Class CompositionOffsetBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class CompositionOffsetBox: public FullBox
{
    public:
        CompositionOffsetBox(unsigned_int8 version = 0);
        virtual ~CompositionOffsetBox();
        
    public:
        unsigned_int32 entry_count;
        Vector<unsigned_int32> sample_count;
        Vector<unsigned_int32> sample_offset;

    public:
        
      /**
        *   Decoded differentially encoded CompositionOffsetBox::sample_offset in a linear array, one entry corresponding to each sample
        *
        *   @see    CompositionOffsetBox::decode_sample_offset
        *
        */
        Vector<unsigned_int32> decoded_sample_offset;
      
      /**
        *   The minimum value in the decoded CompositionOffsetBox::sample_offset
        *
        *   @see    CompositionOffsetBox::decode_sample_offset
        *
        */
        unsigned_int32 min_sample_offset;
      
      /**
        *   The max value in the decoded CompositionOffsetBox::sample_offset
        *
        *   @see    CompositionOffsetBox::decode_sample_offset
        *
        */
        unsigned_int32 max_sample_offset;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add a new CompositionOffsetBox::sample_offset
        *
        *   @return The new CompositionOffsetBox::entry_count
        *
        */
        UInt32 add_sample_offset(unsigned_int32 new_sample_offset);

      /**
        *   Decode the differentially encoded CompositionOffsetBox::sample_offset to linear array of decoded_sample_offset, one entry corresponding to each sample
        *
        *   @see    CompositionOffsetBox::decode_sample_offset
        *
        */
        UInt32 decode_sample_offset();
};


/**
  *   ISO/IEC 14496-12:2005(E) Class SampleEntry, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class SampleEntry: public Box
{
    public:
        SampleEntry(unsigned_int32 format);
        virtual ~SampleEntry();
        
    public:
        static const unsigned_int8 reserved_arr[6];
        unsigned_int16 data_reference_index;

    private:
        virtual void dummyFunc() = 0;
};


/**
  *   ISO/IEC 14496-12:2005(E) Class VisualSampleEntry, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class VisualSampleEntry: public SampleEntry
{
    public:
        VisualSampleEntry(unsigned_int32 codingname);
        virtual ~VisualSampleEntry();
        
    public:
        static const unsigned_int8 reserved_arr[6];
        unsigned_int16 data_reference_index;

    public:
        unsigned_int16 pre_defined;
        static const unsigned_int16 reserved;
        unsigned_int32 pre_defined_arr[3];
        unsigned_int16 width;
        unsigned_int16 height;
        unsigned_int32 horizresolution;
        unsigned_int32 vertresolution;
        static const unsigned_int32 reserved_2;
        unsigned_int16 frame_count;
        string compressorname;
        unsigned_int16 depth;
        int16 pre_defined_2;

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set VisualSampleEntry::compressorname
        *
        */
        void set_compressorname(string name);

    private:
        virtual void dummyFunc(){}
};


/**
  *   ISO/IEC 14496-12:2005(E) Class SampleDescriptionBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class SampleDescriptionBox: public FullBox
{
    public:
        SampleDescriptionBox(UInt32 handler_type_index = 1);
        virtual ~SampleDescriptionBox();
        
    public:
        unsigned_int32 handler_type;
        unsigned_int32 entry_count;
        Vector<SampleEntry *> sampleEntry;

    public:
        unsigned_int32 handler_types[3];

    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Set SampleDescriptionBox::setHandlerType
        *
        */
        void setHandlerType(UInt32 handler_type_index);
        
      /**
        *   Add a new implementation of abstract SampleEntry
        *
        */
        SampleEntry * addNewSampleEntry(unsigned_int32 handler_type);

};


/**
  *   ISO/IEC 14496-12:2005(E) Class SampleToChunkBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class SampleSizeBox: public FullBox 
{
    public:
        SampleSizeBox(unsigned_int8 version = 0);
        virtual ~SampleSizeBox();
        
    public:
        unsigned_int32 sample_size;
        unsigned_int32 sample_count;
        Vector<unsigned_int32> entry_size;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add a new SampleSizeBox::entry_size
        *
        *   @return The new SampleSizeBox::sample_size
        *
        */
        UInt32 add_sample_size(unsigned_int32 new_sample_size);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class SampleToChunkBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class SampleToChunkBox: public FullBox 
{
    public:
        SampleToChunkBox(unsigned_int8 version = 0);
        virtual ~SampleToChunkBox();
        
    public:
        unsigned_int32 entry_count;
        Vector<unsigned_int32> first_chunk;
        Vector<unsigned_int32> samples_per_chunk;
        Vector<unsigned_int32> sample_description_index;
        
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
        
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add a new "entry" consisting of first_chunk, samples_per_chunk, sample_description_index
        *
        *   @return The new SampleToChunkBox::entry_count
        *
        */
        UInt32 add_entry(unsigned_int32 first_chunk,unsigned_int32 samples_per_chunk,unsigned_int32 sample_description_index);
};


/**
  *   ISO/IEC 14496-12:2005(E) Class ChunkOffsetBox, for details about all public undocumented data (attributes), see ISO/IEC 14496-12:2005(E). Such attributes are syntax elements documented in standard under same name and class.
  *   
  */
class ChunkOffsetBox: public FullBox
{
    public:
        ChunkOffsetBox(unsigned_int8 version = 0);
        virtual ~ChunkOffsetBox();
        
    public:
        unsigned_int32 entry_count;
        Vector<unsigned_int32> chunk_offset;
    
    public:
        
      /**
        *   @see    Box::decode
        *
        */
        void decode(Byte *buffer, UInt32 count);
               
      /**
        *   @see    Box::encode
        *
        */
        void encode(Byte **buffer, UInt32 *count);

      /**
        *   @see    Box::estimateSize
        *
        */
        UInt32 estimateSize();

      /**
        *   Add a new ChunkOffsetBox::chunk_offset
        *
        *   @return The new ChunkOffsetBox::entry_count
        *
        */
        UInt32 add_chunk_offset(unsigned_int32 chunk_offset);

};

END_NAMESPACE(ISOFile_Include_IsoFileClasses_hpp);

#endif  //_ISOFILE_INCLUDE_ISO_FILE_CLASSES_HPP_

