// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 15 June, 2006
//

#ifndef _ISOFILE_INCLUDE_ISO_YUV_FILE_HPP_
#define _ISOFILE_INCLUDE_ISO_YUV_FILE_HPP_

#include "Namespace.hpp"
#include "DataTypes.hpp"
#include "IsoFileClasses.hpp"
#include "ClassContainer.hpp"

DECLARE_NAMESPACE(ISOFile_Include_IsoYUVFile_hpp);

using namespace ISOFile_Include_ClassContainer_hpp;

/**
  *     Class IsoYUVFile, provides a high abstraction level API for creating ISO YUV files. Creates the entire header according to ISO/IEC 14496-12:2005(E)
  *     Usage:
  *         1. Create, initialize
  *         2. Add YUV samples (only refernce is added to YUV frame offsets in the header).
  *         3. Finalize (updates header tables)
  *         4. Use ClassContainer::encodeAll to encode the header in a BufferManager object
  *         5. Append the actual YUV raw data to the header to get ISO YUV file
  *   
  */
class IsoYUVFile: public ClassContainer
{
    public:
        
      /**
        *   Class Constructor. init must be used before adding samples/finalizing
        *
        *   @see    IsoYUVFile::init
        *
        */
        IsoYUVFile();
      
      /**
        *   Class Constructor. calls init with these parameters
        *
        *   @param  height      Height of the YUV sequence
        *   @param  width       Width of the YUV sequence
        *   @param  timeScale   timeScale of the YUV sequence, i.e. # of clock ticks in a second
        *
        *   @see    IsoYUVFile::init
        *
        */
        IsoYUVFile(UInt32 height, UInt32 width, UInt32 timeScale);
      
        virtual ~IsoYUVFile();
        
    public:
        unsigned_int32 trackID;
        
    public:
        
      /**
        *   Initialization function. Use only once to create file
        *
        *   @param  height              Height of the YUV sequence
        *   @param  width               Width of the YUV sequence
        *   @param  timeScale           timeScale of the YUV sequence, i.e. # of clock ticks in a second
        *   @param  useSpecifiedTime    If true, the last field is used to initialize header time stamps, else, time is calculated internally using standard C time routines. False by default
        *   @param  specifiedTime       If useSpecifiedTime is true, should carry the time to initialize headers, according to ISO/IEC 14496-12:2005(E). Ignored otherwise
        *
        *   @see    IsoYUVFile::init
        *
        */
        void init(UInt32 height, UInt32 width, UInt32 timeScale, bool useSpecifiedTime = false, UInt32 specifiedTime = 0);

      /**
        *   Add a YUV frame sample reference in the header
        *
        *   @param  size                Number of bytes of current YUV frame
        *   @param  timeDelta           Duration from the last sample
        *   @param  useSpecifiedTime    If true, the last field is used to initialize header time stamps, else, time is calculated internally using standard C time routines. False by default
        *   @param  specifiedTime       If useSpecifiedTime is true, should carry the time to initialize headers, according to ISO/IEC 14496-12:2005(E). Ignored otherwise
        *
        *   @see    IsoYUVFile::init
        *
        */
        void addNewSampleInformation(UInt32 size, UInt32 timeDelta, bool useSpecifiedTime = false, UInt32 specifiedTime = 0);

      /**
        *   After adding all samples, call this to update all the header tables finally. Can be called multiple times to re-finalize after adding some more samples
        *
        *   @param  requiredSize    If non-zero, FreeSpaceBox is used to stuff the header to required size.
        *
        */
        void finalizeHeader(UInt32 requiredSize = 0);

      /**
        *   Can be used to update ChunkOffsetBox entries with a given offset
        *
        */
        void updateChunkOffset(UInt32 offsetToAdd);

    private:
        UInt32 accumulatedOffset;
        
};

END_NAMESPACE(ISOFile_Include_IsoYUVFile_hpp);

#endif //_ISOFILE_INCLUDE_ISO_YUV_FILE_HPP_


