// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#ifndef _ISOFILE_INCLUDE_VECTOR_HPP_
#define _ISOFILE_INCLUDE_VECTOR_HPP_

#include <stdlib.h>
#include "Namespace.hpp"
#include "DataTypes.hpp"

DECLARE_NAMESPACE(ISOFile_Include_Vector_hpp);

using namespace ISOFile_Include_DataTypes_hpp;

  /**
    *   Class Vector, only provides basic functionaly to autmatically allocate more memory whenever required. At the time no advaced sort functions etc. required.
    *   
    */
template <class T> class Vector
{
    public:
      /**
        *   Class constructor with preferred parameters
        *
        *   @param  preferredAllocSize      The allocation size initially
        *   @param  preferredAllocChunkSize The chunk size used for each re-allocation
        *
        *   @return
        */
        Vector(UInt32 preferredAllocSize = defaultAllocChunkSize, UInt32 preferredAllocChunkSize = defaultAllocChunkSize)
        {
            setAllocChunkSize(preferredAllocChunkSize);
            allocSize = 0;
            dataSize = 0;
            data = NULL;
            arrangeMemory(preferredAllocSize);
        }
        
      /**
        *   Class destructor
        */
        ~Vector()
        {
            if(allocSize>0)
                free(data);

            allocSize = 0;
            dataSize = 0;
            data = NULL;
        }

    private:
        UInt32 allocSize;
        UInt32 dataSize;
        UInt32 allocChunkSize;
        static UInt32 defaultAllocChunkSize;
        T *data;

    public:
        
      /**
        *   Access operator for both read and write operation.
        *   
        *   Access operator for both read and write operation. Allocates more if access is out of bound
        *   No difference was observed specifying const qualifier for read operation, atleast MSVC cannot differentiate between the two.
        *
        *   @param  location   Where to read and/or write
        *
        *   @return Return by reference where to read and/or write
        */
        T& operator[](UInt32 location)
        {
            if(location >= allocSize)
                arrangeMemory(allocSize + allocChunkSize);

            if(location >= dataSize)
                dataSize = location+1;

            return data[location];
        }

      /**
        *   Add directly bytes of data provided into storage.
        *   
        *   @param  dataToAdd   Address of location from where data is to be added
        *   @param  count       Number of bytes to be added
        *
        */
        void addData(T* dataToAdd, UInt32 count)
        {
            if(dataToAdd == NULL)
                throw "Vector::addData: Null pointer exception!";
            
            if(count + dataSize > allocSize)
                arrangeMemory(count + dataSize);

            dataSize += count;

            memcpy(data,dataToAdd,count*sizeof(T));
        }
                
      /**
        *   Set the preferred chunk allocation size
        *
        *   @param  preferredAllocChunkSize   The new preferred size of allocation
        *
        */
        void setAllocChunkSize(UInt32 preferredAllocChunkSize)
        {
            if(preferredAllocChunkSize == 0)
                throw "Vector::setAllocChunkSize Illegal chunk size";

            allocChunkSize = preferredAllocChunkSize;
        }
        
      /**
        *   Get the allocated size of memory allocated
        *
        */
        UInt32 getStorageSize()
        {
            return allocSize;
        }
        
      /**
        *   Get the actualy size of stored data
        *
        */
        UInt32 getSize()
        {
            return dataSize;
        }
        
      /**
        *   Set the stored data size to new value
        *
        */
        UInt32 setSize(UInt32 newSize)
        {
            dataSize = newSize < allocSize ? newSize : allocSize;

            return dataSize;
        }
       
      /**
        *   Set the storage size to a new value, more memory might be allocated in the process
        *   
        *   @param  size    Required new size
        *
        */
        void setStorageSize(UInt32 size)
        {
            arrangeMemory(size);
            setSize(size);
        }
        
      /**
        *   Return the actual storage area of data along with the size of the data contained.
        *   
        *   @param  storage Place where the addess of the storage is saved
        *
        *   @return The data size in the storage
        */
        UInt32 getStorage(T **storage)
        {
            if(storage == NULL)
				throw "Vector::getStorage Null pointer exception!";

            *storage = data;

            return getSize();
        }

    private:

      /**
        *   Arrange the required memory. Realloc is used to provide easy move inside the memory
        *   
        *   @param  minimumReqSize  The minimum size of the required memory. Actual allocation might be larger than this depending on the chunk size specified for allication
        *
        */
        void arrangeMemory(UInt32 minimumReqSize)
        {
            allocSize = allocSize + allocChunkSize < minimumReqSize ? minimumReqSize : allocSize + allocChunkSize;

            data = (T *)realloc(data,sizeof(T)*allocSize);
            if(data == NULL)
                throw "Vector::arrangeMemory Memeory Allocation Failure!";
        }
};

template <class T> UInt32 Vector<T>::defaultAllocChunkSize = 16;    //The default size

END_NAMESPACE(ISOFile_Include_Vector_hpp);

#endif  //_ISOFILE_INCLUDE_VECTOR_HPP_

