// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//


#define _ISOFILE_BUFFER_MANAGER_CPP_

#include <memory.h>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include "BufferManager.hpp"

using namespace ISOFile_Include_BufferManager_hpp;

UInt32 BufferManager::defaultAllocChunkSize = 16;    //The default size

BufferManager::BufferManager(UInt32 preferredAllocSize, UInt32 preferredAllocChunkSize)
{
    unsigned int endianTest = 0x01020304;
    if(((unsigned char *)&endianTest)[0] != 0x04)
        throw "BufferManager: Big endian not supported";
    
    setReadPointer(0);
    setWritePointer(0);
    maxNetworkWritePointer = 0;
    networkISOFileData.setAllocChunkSize(preferredAllocChunkSize);
}

BufferManager::~BufferManager()
{    
    setReadPointer(0);
    setWritePointer(0);
    maxNetworkWritePointer = 0;
}

UInt32 BufferManager::getBuffer(Byte **buffer)
{
    if(buffer == NULL)
        throw "BufferManager::getBuffer: NULL pointer exception";
    
    return networkISOFileData.getStorage(buffer);
}

UInt32 BufferManager::incrementWritePointer(UInt32 increment)
{
    setWritePointer(networkWritePointer+increment);

    //This will save the most relaible account of uptil where the last data has been written. Note that read and write pointers can be set arbitrarily
    if(networkWritePointer > maxNetworkWritePointer)
        maxNetworkWritePointer = networkWritePointer;
    
    return networkWritePointer;
}

UInt32 BufferManager::getWritePointer()
{
    return networkWritePointer;
}

UInt32 BufferManager::setWritePointer(UInt32 newWritePointer)
{
    networkWritePointer = newWritePointer;

    return networkWritePointer;
}

UInt32 BufferManager::incrementReadPointer(UInt32 increment)
{
    setReadPointer(networkReadPointer+increment);
    
    return networkReadPointer;
}

UInt32 BufferManager::getReadPointer()
{
    return networkReadPointer;
}

UInt32 BufferManager::setReadPointer(UInt32 newReadPointer)
{
    //networkReadPointer = networkISOFileData.getSize() > 0 ? (newReadPointer < networkISOFileData.getSize() ? newReadPointer : networkISOFileData.getSize() - 1) : 0;
    networkReadPointer = newReadPointer;
    
    return networkReadPointer;
}

UInt32 BufferManager::writeData(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "BufferManager::writeData: NULL pointer exception";
    
    if(sizeof(Byte) != 1)
        throw "BufferManager::writeByte: Improper Byte size exception";

    UInt32 minimumNewStorageSize = networkWritePointer + sizeof(Byte)*count;
    
    if(networkISOFileData.getStorageSize()< minimumNewStorageSize)
        networkISOFileData.setStorageSize(minimumNewStorageSize);

    memcpy(&networkISOFileData[networkWritePointer],buffer,sizeof(Byte)*count);

    if(minimumNewStorageSize > networkISOFileData.getSize())
        networkISOFileData.setSize(minimumNewStorageSize);
    
    return incrementWritePointer(sizeof(Byte)*count);
}

UInt32 BufferManager::writeByte(Byte data)
{
    if(sizeof(Byte) != 1)
        throw "BufferManager::writeByte: Improper Byte size exception";

    networkISOFileData[networkWritePointer] = data;
    
    return incrementWritePointer(sizeof(Byte));
}

UInt32 BufferManager::writeUInt16(UInt16 data)
{
    if(sizeof(UInt16) != 2)
        throw "BufferManager::writeUInt16: Improper UInt16 size exception";
    
    UInt32 minimumNewStorageSize = networkWritePointer + sizeof(UInt16);
    
    if(networkISOFileData.getStorageSize() < minimumNewStorageSize)
        networkISOFileData.setStorageSize(minimumNewStorageSize);

    UInt16 networkOrderUInt16 = (data << 8) | (data >> 8);//Reverse the bytes to Network or Big-endian order

    *((UInt16 *)&(networkISOFileData[networkWritePointer])) = networkOrderUInt16;
    
    if(minimumNewStorageSize > networkISOFileData.getSize())
        networkISOFileData.setSize(minimumNewStorageSize);
    
    return incrementWritePointer(sizeof(UInt16));
}

UInt32 BufferManager::writeUInt32(UInt32 data)
{
    if(sizeof(UInt32) != 4)
        throw "BufferManager::writeUInt32: Improper UInt32 size exception";
    
    UInt32 minimumNewStorageSize = networkWritePointer + sizeof(UInt32);
    
    if(networkISOFileData.getStorageSize() < minimumNewStorageSize)
        networkISOFileData.setStorageSize(minimumNewStorageSize);

    UInt32 networkOrderUInt32 = (data << 24) | ((data & 0xFF00) << 8) | ((data & 0xFF0000) >> 8) | (data >> 24);//Reverse the bytes to Network or Big-endian order

    *((UInt32 *)&(networkISOFileData[networkWritePointer])) = networkOrderUInt32;
    
    if(minimumNewStorageSize > networkISOFileData.getSize())
        networkISOFileData.setSize(minimumNewStorageSize);
    
    return incrementWritePointer(sizeof(UInt32));
}

UInt32 BufferManager::writebit(UInt32 bitCount, UInt32 data)
{
    if(bitCount%8 != 0)
        throw "BufferManager::writebit: Only Byte aligned write supported!";
    
    if(bitCount > 32 || bitCount == 0)
        throw "BufferManager::writebit: Improper number of bits to written!";

    UInt32 byteCount = bitCount/8;
    UInt32 bitShift = bitCount - 8;
        
    UInt32 minimumNewStorageSize = networkWritePointer + byteCount;
    
    if(networkISOFileData.getStorageSize() < minimumNewStorageSize)
        networkISOFileData.setStorageSize(minimumNewStorageSize);
    
    for(UInt32 index = 0 ; index < byteCount ; index++ , bitShift -= 8)
    {
        networkISOFileData[networkWritePointer+index] = (data >> bitShift) & 0xFF;
    }
    
    if(minimumNewStorageSize > networkISOFileData.getSize())
        networkISOFileData.setSize(minimumNewStorageSize);
    
    return incrementWritePointer(byteCount);
}

UInt32 BufferManager::readData(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "BufferManager::readData: NULL pointer exception";
    
    if(sizeof(Byte) != 1)
        throw "BufferManager::readData: Improper Byte size exception";

    if(networkISOFileData.getSize() < networkReadPointer + count)
        throw "BufferManager::readData: Read Out of range!";

    memcpy(buffer,&networkISOFileData[networkReadPointer],count);
    
    return incrementReadPointer(count);
}

UInt32 BufferManager::readByte(Byte* data)
{
    if(sizeof(Byte) != 1)
        throw "BufferManager::readByte: Improper Byte size exception";
    
    if(data == NULL)
        throw "BufferManager::readByte: NULL pointer exception";
  
    if(networkISOFileData.getSize() < networkReadPointer + sizeof(Byte))
        throw "BufferManager::readByte: Read Out of range!";

    *data = networkISOFileData[networkReadPointer];
    
    return incrementReadPointer(sizeof(Byte));
}

UInt32 BufferManager::readUInt16(UInt16* data)
{
    if(sizeof(UInt16) != 2)
        throw "BufferManager::readUInt16: Improper UInt16 size exception";
    
    if(data == NULL)
        throw "BufferManager::readUInt16: NULL pointer exception";
        
    if(networkISOFileData.getSize() < networkReadPointer + sizeof(UInt16))
        throw "BufferManager::readUInt16: Read Out of range!";

    UInt16 networkOrderUInt16 = *((UInt16 *)&(networkISOFileData[networkReadPointer]));
    
    *data = (networkOrderUInt16 << 8) | (networkOrderUInt16 >> 8);  //Reverse the bytes to little-endian order
    
    return incrementReadPointer(sizeof(UInt16));
}

UInt32 BufferManager::readUInt32(UInt32* data)
{
    if(sizeof(UInt32) != 4)
        throw "BufferManager::readUInt32: Improper UInt32 size exception";
    
    if(data == NULL)
        throw "BufferManager::readUInt32: NULL pointer exception";
    
    if(networkISOFileData.getSize() < networkReadPointer + sizeof(UInt32))
    {
         throw "BufferManager::readUInt32: Read Out of range!";
    }

    UInt32 networkOrderUInt32 = *((UInt32 *)&(networkISOFileData[networkReadPointer]));

    *data = (networkOrderUInt32 << 24) | ((networkOrderUInt32 & 0xFF00) << 8) | ((networkOrderUInt32 & 0xFF0000) >> 8) | (networkOrderUInt32 >> 24); //Reverse the bytes to little-endian order
    
    return incrementReadPointer(sizeof(UInt32));
}

UInt32 BufferManager::readbit(UInt32 bitCount, UInt32* data)
{
    if(bitCount%8 != 0)
        throw "BufferManager::readbit: Only Byte aligned read supported!";
    
    if(bitCount > 32 || bitCount == 0)
        throw "BufferManager::readbit: Improper number of bits to read!";

    UInt32 byteCount = bitCount/8;
    UInt32 bitShift = bitCount - 8;
    
    if(data == NULL)
        throw "BufferManager::readbit: NULL pointer exception";
    
    if(networkISOFileData.getSize() < networkReadPointer + byteCount)
        throw "BufferManager::readbit: Read Out of range!";

    *data = 0x0;

    for(UInt32 index = 0 ; index < byteCount ; index++ , bitShift -= 8)
    {
        *data |= (networkISOFileData[networkReadPointer+index] << bitShift);
    }

    return incrementReadPointer(byteCount);
}

