// Software developed by BenQ Mobile GmbH & Co. OHG for support of simulation framework within 3GPP SA4
//
// Waqar Zia (waqar.zia@benq.com) 27 May, 2006
//

#define _ISOFILE_ISO_FILE_CLASSES_CPP_

#ifdef WIN32
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <time.h>
#include <stdio.h>
#include "IsoFileClasses.hpp"
#include "IsoBoxTypes.hpp"

using namespace ISOFile_Include_IsoFileClasses_hpp;
using namespace ISOFile_Include_IsoBoxTypes_hpp;

Box::Box()
{
    allocBufManager();
}

Box::Box(const char *boxtype)
{
    allocBufManager();
    set_type(boxtype);
}

Box::Box(unsigned_int32 boxtype)
{
    allocBufManager();
    type = boxtype;
}

Box::Box(unsigned_int32 boxtype, unsigned_int8 extended_type[16])
{
    throw "Box: extended_type not supported!";
}

Box::~Box()
{
    delete bufManager;
}

void Box::allocBufManager()
{
    bufManager = new BufferManager((UInt32)2*sizeof(unsigned_int32));
    if(bufManager == NULL)
        throw "Box::allocBufManager: Allocation failure!";
}

void Box::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "Box::decode: NULL pointer exception";
    
    bufManager->writeData(buffer, count);

    bufManager->setReadPointer(0);

    bufManager->readUInt32(&size);

    //Bail out right now, proper decoding not assured!
    if(size > count)
	{
		int reqSize = size;
        throw (reqSize);
	}

    bufManager->readUInt32(&type);
        
}

void Box::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "Box::encode: NULL pointer exception";
    
    bufManager->setWritePointer(0);

    bufManager->writeUInt32(size);
        
    bufManager->writeUInt32(type);

    update_size();

    *count = getBuffer(buffer);
}

UInt32 Box::estimateSize()
{
    return 2*sizeof(unsigned_int32);
}

void Box::set_type(const char *str)
{
    if(str == NULL)
        throw "Box::set_type: NULL pointer exception";

    mapStringToUInt32((char *)str,&type);

}

void Box::mapStringToUInt32(char *src, unsigned_int32 *target)
{
    if(src == NULL || target == NULL)
        throw "Box::mapStringToUInt32: NULL pointer exception";

    char *tmp = (char *)target;

    //Write in reverse (big-endian) format)

    tmp[0] = src[3];
    tmp[1] = src[2];
    tmp[2] = src[1];
    tmp[3] = src[0];

}

void Box::mapUInt32ToString(unsigned_int32 *src, char *target)
{
    if(src == NULL || target == NULL)
        throw "Box::mapStringToUInt32: NULL pointer exception";

    char *tmp = (char *)src;

    //Write in reverse (big-endian) format)

    target[0] = tmp[3];
    target[1] = tmp[2];
    target[2] = tmp[1];
    target[3] = tmp[0];
    target[4] = '\0';
}

void Box::update_size()
{
    size = getCurrentWritePointer();
    write_size();
}

void Box::write_size()
{
    UInt32 savedWritePtr = getCurrentWritePointer();
    bufManager->setWritePointer(0);
    bufManager->writeUInt32(size);
    bufManager->setWritePointer(savedWritePtr);
}

UInt32 Box::getCurrentWritePointer()
{
    return bufManager->getWritePointer();
}

UInt32 Box::getCurrentReadPointer()
{
    return bufManager->getReadPointer();
}

UInt32 Box::getBuffer(Byte **buffer)
{
    if(buffer == NULL)
        throw "Box::getBuffer: NULL pointer exception";
    
    return bufManager->getBuffer(buffer);
}

FullBox::FullBox()
{
}

FullBox::FullBox(const char *boxtype, unsigned_int8 v, bit24 f):Box(boxtype)
{
    this->version = v;
    this->flags = f;
}

FullBox::FullBox(unsigned_int32 boxtype, unsigned_int8 v, bit24 f):Box(boxtype)
{
    this->version = v;
    this->flags = f;
}

FullBox::~FullBox()
{
}

void FullBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "FullBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    bufManager->readByte(&version);
        
    bufManager->readbit(24,&flags);
}

void FullBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "FullBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
   
    bufManager->writeByte(version);
        
    bufManager->writebit(24,flags);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 FullBox::estimateSize()
{
    return Box::estimateSize() + sizeof(unsigned_int8) + 24/8;
}

FileTypeBox::FileTypeBox():Box(IsoBoxTypes::type_FileTypeBox)
{
}

FileTypeBox::~FileTypeBox()
{
}

void FileTypeBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "FileTypeBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    bufManager->readUInt32(&major_brand);
        
    bufManager->readUInt32(&minor_version);

    for(UInt32 index = 0; getCurrentReadPointer() < count-1 ; index++)
    {
        bufManager->readUInt32(&compatible_brands[index]);
    }

}

void FileTypeBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "FullBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
   
    bufManager->writeUInt32(major_brand);
        
    bufManager->writeUInt32(minor_version);

    for(UInt32 index = 0 ; index < compatible_brands.getSize(); index++)
    {
        bufManager->writeUInt32(compatible_brands[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 FileTypeBox::estimateSize()
{
    return Box::estimateSize() + (2+compatible_brands.getSize())*sizeof(unsigned_int32);
}

MovieBox::MovieBox():Box(IsoBoxTypes::type_MovieBox)
{
}

MovieBox::~MovieBox()
{
}

void MovieBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MovieBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void MovieBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MovieBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 MovieBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* MovieBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

MediaDataBox::MediaDataBox():Box(IsoBoxTypes::type_MediaDataBox)
{
}

MediaDataBox::~MediaDataBox()
{
}

void MediaDataBox::addData(bit8* dataToAdd, UInt32 count)
{
    data.addData(dataToAdd,count);
}

void MediaDataBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MediaDataBox::decode: NULL pointer exception";

     Box::decode(buffer,count);

    UInt32 dataSize = size > 0 ? size : count - getCurrentReadPointer();
 
    for(UInt32 index = 0 ; index < dataSize ; index++)
    {         
        bufManager->readByte(&data[index]);
    }
 }

void MediaDataBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MediaDataBox::encode: NULL pointer exception";

    bool tillEndOfFile = size == 0;

    Box::encode(buffer,count);
   
    UInt32 dataSize = !tillEndOfFile ? size : data.getSize();

    for(UInt32 index = 0 ; index < dataSize ; index++)
        bufManager->writeByte(data[index]);

    if(tillEndOfFile)
        size = 0;
    
    write_size();
    
    *count = getBuffer(buffer);
}

UInt32 MediaDataBox::estimateSize()
{
    return Box::estimateSize() + data.getSize()*sizeof(unsigned_int8);
}

FreeSpaceBox::FreeSpaceBox():Box(IsoBoxTypes::type_FreeSpaceBox)
{
}

FreeSpaceBox::~FreeSpaceBox()
{
}

void FreeSpaceBox::addData(bit8* dataToAdd, UInt32 count)
{
    data.addData(dataToAdd,count);
}

void FreeSpaceBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "FreeSpaceBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    UInt32 dataSize = size - getCurrentReadPointer();

    for(UInt32 index = 0 ; index < dataSize ; index++)
        bufManager->readByte(&data[index]);
}

void FreeSpaceBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "FreeSpaceBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
   
    UInt32 dataSize = data.getSize();

    for(UInt32 index = 0 ; index < dataSize ; index++)
        bufManager->writeByte(data[index]);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 FreeSpaceBox::estimateSize()
{
    return Box::estimateSize() + data.getSize()*sizeof(unsigned_int8);
}

const bit16 MovieHeaderBox::reserved = 0;
const unsigned_int32 MovieHeaderBox::reserved_arr[2] = {0,0};

const UInt32 MovieHeaderBox::timeFrom1904To1970 = ((1970 - 1904)*365 + (UInt32)((1970 - 1904)/4))*24*60*60; //==2082758400 seconds

MovieHeaderBox::MovieHeaderBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_MovieHeaderBox, version, 0)
{
    if(version != 0)
        throw "MovieHeaderBox: Only version 0 supported!";
    
    rate = 0x00010000; // typically 1.0
    volume = 0x0100; // typically, full volume
    int32 localMatrix[9] = { 0x00010000,0,0,0,0x00010000,0,0,0,0x40000000 };// Unity matrix
    memcpy(matrix,localMatrix,9*sizeof(int32));
    memset(pre_defined,0,6*sizeof(bit32));
}

MovieHeaderBox::~MovieHeaderBox()
{
}

void MovieHeaderBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MovieHeaderBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&creation_time);
    bufManager->readUInt32(&modification_time);
    bufManager->readUInt32(&timescale);
    bufManager->readUInt32(&duration);
    bufManager->readUInt32((UInt32*)&rate);
    bufManager->readUInt16((UInt16*)&volume);
    
    UInt32 tmp;
    UInt16 tmp16;
   
    bufManager->readUInt16(&tmp16);
    if(tmp16 != reserved)
        throw "MovieHeaderBox::decode: Unexpected reserve value!";

    for(int index = 0 ; index < 2 ; index++)
    {
        bufManager->readUInt32(&tmp);
        if(tmp != reserved_arr[index])
            throw "MovieHeaderBox::decode: Unexpected reserve value!";
    }
    
    for(int index = 0 ; index < 9 ; index++)
       bufManager->readUInt32((UInt32*)&matrix[index]);
    
    for(int index = 0 ; index < 6 ; index++)
        bufManager->readUInt32((UInt32*)&pre_defined[index]);
    
    bufManager->readUInt32(&next_track_ID);
}

void MovieHeaderBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MovieHeaderBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(creation_time);
    bufManager->writeUInt32(modification_time);
    bufManager->writeUInt32(timescale);
    bufManager->writeUInt32(duration);
    bufManager->writeUInt32((UInt32)rate);
    bufManager->writeUInt16((UInt16)volume);
    bufManager->writeUInt16((UInt16)reserved);

    for(int index = 0 ; index < 2 ; index++)
        bufManager->writeUInt32(reserved_arr[index]);
    
    for(int index = 0 ; index < 9 ; index++)
       bufManager->writeUInt32(((UInt32)matrix[index]));
    
    for(int index = 0 ; index < 6 ; index++)
        bufManager->writeUInt32(((UInt32)pre_defined[index]));
    
    bufManager->writeUInt32(next_track_ID);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 MovieHeaderBox::estimateSize()
{
    return FullBox::estimateSize() + 5*sizeof(unsigned_int32) + 2*sizeof(unsigned_int16) + (2+9+6+1)*sizeof(unsigned_int32);
}

unsigned_int32 MovieHeaderBox::getCurrentISOFileFormatTime()
{
    return (unsigned_int32)time(NULL) + timeFrom1904To1970;
}

TrackBox::TrackBox():Box(IsoBoxTypes::type_TrackBox)
{
}

TrackBox::~TrackBox()
{
}

void TrackBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "TrackBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void TrackBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "TrackBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 TrackBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* TrackBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

const unsigned_int32 TrackHeaderBox::reserved = 0;
const unsigned_int32 TrackHeaderBox::reserved_arr[2] = {0,0};
const unsigned_int16 TrackHeaderBox::reserved_2 = 0;

TrackHeaderBox::TrackHeaderBox(unsigned_int8 version, bit24 flags):FullBox(IsoBoxTypes::type_TrackHeaderBox, version, flags)
{
    if(version != 0)
        throw "TrackHeaderBox: Only version 0 supported!";
    
    layer = 0;
    alternate_group = 0;
    set_volume(0);
    int32 localMatrix[9] = { 0x00010000,0,0,0,0x00010000,0,0,0,0x40000000 };
    memcpy(matrix,localMatrix,9*sizeof(int32));
}

TrackHeaderBox::~TrackHeaderBox()
{
}

void TrackHeaderBox::set_volume(UInt32 track_is_audio)
{
    if(track_is_audio)
        volume = 0x0100;
    else
        volume = 0;
}

void TrackHeaderBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "TrackHeaderBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&creation_time);
    bufManager->readUInt32(&modification_time);
    bufManager->readUInt32(&track_ID);

    UInt32 tmp;
    UInt16 tmp16;
    
    bufManager->readUInt32(&tmp);
    if(tmp != reserved)
        throw "TrackHeaderBox::decode: Unexpected reserve value!";

    bufManager->readUInt32(&duration);
    
    for(int index = 0 ; index < 2 ; index++)
    {
        bufManager->readUInt32(&tmp);
        if(tmp != reserved_arr[index])
            throw "MovieHeaderBox::decode: Unexpected reserve value!";
    }

    bufManager->readUInt16((UInt16*)&layer);
    bufManager->readUInt16((UInt16*)&alternate_group);
    bufManager->readUInt16((UInt16*)&volume);
    bufManager->readUInt16(&tmp16);
    if(tmp16 != reserved_2)
        throw "TrackHeaderBox::decode: Unexpected reserve value!";

    for(int index = 0 ; index < 9 ; index++)
       bufManager->readUInt32((UInt32*)&matrix[index]);
    
    bufManager->readUInt32(&width);
    bufManager->readUInt32(&height);
}

void TrackHeaderBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "TrackHeaderBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(creation_time);
    bufManager->writeUInt32(modification_time);
    bufManager->writeUInt32(track_ID);
    bufManager->writeUInt32(reserved);
    bufManager->writeUInt32(duration);
    
    for(int index = 0 ; index < 2 ; index++)
        bufManager->writeUInt32(reserved_arr[index]);

    bufManager->writeUInt16((UInt16)layer);
    bufManager->writeUInt16((UInt16)alternate_group);
    bufManager->writeUInt16((UInt16)volume);
    bufManager->writeUInt16((UInt16)reserved_2);

    for(int index = 0 ; index < 9 ; index++)
       bufManager->writeUInt32((UInt32)matrix[index]);
    
    bufManager->writeUInt32(width);
    bufManager->writeUInt32(height);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 TrackHeaderBox::estimateSize()
{
    return FullBox::estimateSize() + (5+2)*sizeof(unsigned_int32) + 4*sizeof(unsigned_int16) + (9+2)*sizeof(unsigned_int32);
}

MediaBox::MediaBox():Box(IsoBoxTypes::type_MediaBox)
{
}

MediaBox::~MediaBox()
{
}

void MediaBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MediaBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void MediaBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MediaBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 MediaBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* MediaBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

MediaHeaderBox::MediaHeaderBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_MediaHeaderBox, version, 0)
{
    if(version != 0)
        throw "MediaHeaderBox: Only version 0 supported!";
    
    pre_defined = 0;
}

MediaHeaderBox::~MediaHeaderBox()
{
}

void MediaHeaderBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MediaHeaderBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&creation_time);
    bufManager->readUInt32(&modification_time);
    bufManager->readUInt32(&timescale);
    bufManager->readUInt32(&duration);
    
    UInt16 temp;
    bufManager->readUInt16(&temp);
    pad = (temp & 0x8000)>>15;
    language[2] = (temp & 0x7C00)>>10;
    language[1] = (temp & 0x3E0)>>5;
    language[0] = temp & 0x1F;

    bufManager->readUInt16(&temp);
    if(temp != pre_defined)
        throw "TrackHeaderBox::decode: Unexpected reserve value!";

}

void MediaHeaderBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MediaHeaderBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(creation_time);
    bufManager->writeUInt32(modification_time);
    bufManager->writeUInt32(timescale);
    bufManager->writeUInt32(duration);
    
    UInt16 temp = ((pad & 0x1) << 15) | ((language[2] & 0x1F) << 10) | ((language[1] & 0x1F) << 5) | (language[0] & 0x1F);

    bufManager->writeUInt16(temp);
    bufManager->writeUInt16(pre_defined);

    update_size();
    
    *count = getBuffer(buffer);
}

void MediaHeaderBox::setLanguageFromISOCode(char *isoLanguageCode)
{
    if(isoLanguageCode == NULL)
        throw "MediaHeaderBox::setLanguageFromISOCode: NULL pointer exception";
    
    if(isoLanguageCode[0] < 0x60 || isoLanguageCode[1] < 0x60 || isoLanguageCode[2] < 0x60)
        throw "MediaHeaderBox::setLanguageFromISOCode: In-appropriate language code";

    language[2] = isoLanguageCode[0] - 0x60;
    language[1] = isoLanguageCode[1] - 0x60;
    language[0] = isoLanguageCode[2] - 0x60;
}

void MediaHeaderBox::getLanguageFromISOCode(char *isoLanguageCode)
{
    if(isoLanguageCode == NULL)
        throw "MediaHeaderBox::getLanguageFromISOCode: NULL pointer exception";
    
    if(language[0] > (0xFF-0x60) || language[1] > (0xFF-0x60) || language[2] > (0xFF-0x60))
        throw "MediaHeaderBox::getLanguageFromISOCode: In-appropriate language code";

     isoLanguageCode[0] = language[2] + 0x60;
     isoLanguageCode[1] = language[1] + 0x60;
     isoLanguageCode[2] = language[0] + 0x60;
     isoLanguageCode[4] = '\0';
}

UInt32 MediaHeaderBox::estimateSize()
{
    return FullBox::estimateSize() + 4*sizeof(unsigned_int32) + 2*sizeof(unsigned_int16);
}

HandlerBox::HandlerBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_HandlerBox, version, 0)
{
    if(version != 0)
        throw "HandlerBox: Only version 0 supported!";
    
    pre_defined = 0;

    memset(reserved,0,3*sizeof(unsigned_int32));

    name = NULL;
}

HandlerBox::~HandlerBox()
{
    clear_name(); 
}

void HandlerBox::alloc_name(UInt32 size)
{
    clear_name();
    name = new char[size];
}

void HandlerBox::clear_name()
{
    if(name)
        delete [] name; 
}

void HandlerBox::set_name(string new_name)
{
    alloc_name((UInt32)strlen(new_name)+1); //1 extra for null character
    strcpy(name,new_name);
}

void HandlerBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "HandlerBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    UInt32 temp;

    bufManager->readUInt32(&temp);
    if(temp != pre_defined)
        throw "HandlerBox::decode: Unexpected reserve value!";
    
    bufManager->readUInt32(&handler_type);

    for(int index = 0 ; index < 3 ; index++)
    {
        bufManager->readUInt32(&temp);
        if(temp != reserved[index])
            throw "HandlerBox::decode: Unexpected reserved value!";
    }

    UInt32 nameStringSize = size-getCurrentReadPointer();

    alloc_name(nameStringSize);

    bufManager->readData((Byte *)name,nameStringSize);
}

void HandlerBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "HandlerBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(pre_defined);    
    bufManager->writeUInt32(handler_type);

    for(int index = 0 ; index < 3 ; index++)
    {
        bufManager->writeUInt32(reserved[index]);
    }

    bufManager->writeData((Byte *)name,(UInt32)strlen(name)+1);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 HandlerBox::estimateSize()
{
    return FullBox::estimateSize() + (2+3)*sizeof(unsigned_int32) + ((UInt32)strlen(name)+1)*sizeof(Byte);
}

MediaInformationBox::MediaInformationBox():Box(IsoBoxTypes::type_MediaInformationBox)
{
}

MediaInformationBox::~MediaInformationBox()
{
}

void MediaInformationBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "MediaInformationBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void MediaInformationBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "MediaInformationBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 MediaInformationBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* MediaInformationBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

VideoMediaHeaderBox::VideoMediaHeaderBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_VideoMediaHeaderBox, version, 1)
{    
    graphicsmode = 0;

    memset(opcolor,0,3*sizeof(unsigned_int16));
}

VideoMediaHeaderBox::~VideoMediaHeaderBox()
{
}

void VideoMediaHeaderBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "VideoMediaHeaderBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    UInt16 temp;

    bufManager->readUInt16(&temp);
    if(temp != graphicsmode)
        throw "VideoMediaHeaderBox::decode: Unexpected graphicsmode value!";
    
    for(int index = 0 ; index < 3 ; index++)
    {
        bufManager->readUInt16(&temp);
        if(temp != opcolor[index])
            throw "VideoMediaHeaderBox::decode: Unexpected opcolor value!";
    }
}

void VideoMediaHeaderBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "VideoMediaHeaderBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt16(graphicsmode);
    
    for(int index = 0 ; index < 3 ; index++)
    {
        bufManager->writeUInt16(opcolor[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 VideoMediaHeaderBox::estimateSize()
{
    return FullBox::estimateSize() + (1+3)*sizeof(unsigned_int16);
}

DataInformationBox::DataInformationBox():Box(IsoBoxTypes::type_DataInformationBox)
{
}

DataInformationBox::~DataInformationBox()
{
}

void DataInformationBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "DataInformationBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void DataInformationBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "DataInformationBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 DataInformationBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* DataInformationBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

DataEntryUrlBox::DataEntryUrlBox(bit24 flags, unsigned_int8 version):FullBox(IsoBoxTypes::type_DataEntryUrlBox, version, 0)
{
    if(version != 0)
        throw "DataEntryUrlBox: Only version 0 supported!";

    locationSize = 0;
    location = NULL;
}

DataEntryUrlBox::~DataEntryUrlBox()
{
    clear_location(); 
}

void DataEntryUrlBox::alloc_location(UInt32 size)
{
    if(flags == 0x000001)
        throw "DataEntryUrlBox::alloc_location: Flags indicate ONLY local data!";
    
    clear_location();
    location = new char[size];

    locationSize = size;
}

void DataEntryUrlBox::clear_location()
{
    if(location)
        delete [] location;
    
    locationSize = 0;
}

void DataEntryUrlBox::set_location(string new_location)
{
    if(new_location == NULL)
        throw "DataEntryUrlBox::set_location: NULL pointer exception";
    
    alloc_location((UInt32)strlen(new_location)+1); //1 extra for null character
    
    strcpy(location,new_location);
}

void DataEntryUrlBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "DataEntryUrlBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    locationSize = size-getCurrentReadPointer();

    if(locationSize > 0)
    {
        if(buffer[count-1] != '\0')
            throw "DataEntryUrlBox::decode: Improper location string!";
        
        alloc_location(locationSize);
        bufManager->readData((Byte *)location,locationSize);
    }
}

void DataEntryUrlBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "DataEntryUrlBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);

    if(locationSize > 0)
        bufManager->writeData((Byte *)location,locationSize);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 DataEntryUrlBox::estimateSize()
{
    return FullBox::estimateSize() + locationSize*sizeof(Byte);
}

DataEntryUrnBox::DataEntryUrnBox(bit24 flags, unsigned_int8 version):FullBox(IsoBoxTypes::type_DataEntryUrnBox, version, 0)
{
    if(version != 0)
        throw "DataEntryUrnBox: Only version 0 supported!";
    
    nameSize = 0;
    name = NULL;

    locationSize = 0;
    location = NULL;
}

DataEntryUrnBox::~DataEntryUrnBox()
{
    clear_name(); 
    clear_location(); 
}

void DataEntryUrnBox::alloc_location(UInt32 size)
{
    if(flags == 0x000001)
        throw "DataEntryUrnBox::alloc_location: Flags indicate ONLY local data!";
    
    clear_location();
    location = new char[size];

    locationSize = size;
}

void DataEntryUrnBox::clear_location()
{
    if(location)
        delete [] location;
    
    locationSize = 0;
}

void DataEntryUrnBox::set_location(string new_location)
{
    if(new_location == NULL)
        throw "DataEntryUrnBox::set_location: NULL pointer exception";

    if(flags == 0x000001)
        throw "DataEntryUrnBox::set_location: Flags indicate ONLY loacal data!";
    
    alloc_location((UInt32)strlen(new_location)+1); //1 extra for null character
    
    strcpy(location,new_location);
}

void DataEntryUrnBox::alloc_name(UInt32 size)
{
    if(flags == 0x000001)
        throw "DataEntryUrlBox::alloc_name: Flags indicate ONLY local data!";
    
    clear_name();
    name = new char[size];

    nameSize = size;
}

void DataEntryUrnBox::clear_name()
{
    if(name)
        delete [] name;
    
    nameSize = 0;
}

void DataEntryUrnBox::set_name(string new_name)
{
    if(new_name == NULL)
        throw "DataEntryUrnBox::set_name: NULL pointer exception";

    if(flags == 0x000001)
        throw "DataEntryUrnBox::set_name: Flags indicate ONLY loacal data!";
    
    alloc_name((UInt32)strlen(new_name)+1); //1 extra for null character
    
    strcpy(name,new_name);
}

void DataEntryUrnBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "DataEntryUrnBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    UInt32 stringTotalSize = size-getCurrentReadPointer();
 
    if(stringTotalSize > 0)
    {
        if(buffer[count-1] != '\0')
            throw "DataEntryUrnBox::decode: Improper location/name string!";

        char *temp = new char[stringTotalSize];

        sscanf((const char *)&buffer[getCurrentReadPointer()],"%s",temp);

        alloc_name((UInt32)strlen(temp)+1);

        bufManager->readData((Byte *)name,nameSize);

        if(stringTotalSize - nameSize > 0)
        {
            alloc_location(stringTotalSize - nameSize);
            bufManager->readData((Byte *)location,locationSize);    
        }
        
        delete [] temp;
    }
}

void DataEntryUrnBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "DataEntryUrnBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);

    if(locationSize > 0)
        bufManager->writeData((Byte *)location,locationSize);

    if(nameSize > 0)
        bufManager->writeData((Byte *)name,nameSize);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 DataEntryUrnBox::estimateSize()
{
    return FullBox::estimateSize() + locationSize*sizeof(Byte) + nameSize*sizeof(Byte);
}


DataReferenceBox::DataReferenceBox(bit24 flags, unsigned_int8 version):FullBox(IsoBoxTypes::type_DataReferenceBox, version, 0)
{
    if(version != 0)
        throw "DataReferenceBox: Only version 0 supported!";

    entry_count = 0;
}

DataReferenceBox::~DataReferenceBox()
{
}

Box* DataReferenceBox::addNewBox(const char* type)
{
    Box *newBox = ClassContainer::addNewBox(type);
    
    entry_count = boxes.getSize();

    return newBox;
}

void DataReferenceBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "DataReferenceBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);
    
    ClassContainer::decodeAll(bufManager);

    if(boxes.getSize() != entry_count)
        throw "DataReferenceBox::decode: Different number of DataEntryBox's encountered than expected!";
}

void DataReferenceBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "DataReferenceBox::encode: NULL pointer exception";

    if(boxes.getSize() != entry_count)
        throw "DataReferenceBox::encode: Different number of DataEntryBox's to encode than expected!";

    FullBox::encode(buffer,count);
    
    bufManager->writeUInt32(entry_count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 DataReferenceBox::estimateSize()
{
    return FullBox::estimateSize() + sizeof(unsigned_int32) + ClassContainer::estimateMemberSize();
}

SampleTableBox::SampleTableBox():Box(IsoBoxTypes::type_DataInformationBox)
{
}

SampleTableBox::~SampleTableBox()
{
}

void SampleTableBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "SampleTableBox::decode: NULL pointer exception";

    Box::decode(buffer,count);

    ClassContainer::decodeAll(bufManager);
}

void SampleTableBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "SampleTableBox::encode: NULL pointer exception";

    Box::encode(buffer,count);
    
    ClassContainer::encodeAll(bufManager);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 SampleTableBox::estimateSize()
{
    return Box::estimateSize() + ClassContainer::estimateMemberSize();
}

Box* SampleTableBox::getBoxRecursive(const char* type)
{
    return ClassContainer::getBoxRecursive(type);
}

TimeToSampleBox::TimeToSampleBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_TimeToSampleBox, version, 0)
{
    if(version != 0)
        throw "TimeToSampleBox: Only version 0 supported!";    

    min_sample_delta = (UInt32)-1;
    max_sample_delta = 0;
    entry_count = 0;
}

TimeToSampleBox::~TimeToSampleBox()
{
}

void TimeToSampleBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "TimeToSampleBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->readUInt32(&sample_count[index]);
        bufManager->readUInt32(&sample_delta[index]);
    }
}

UInt32 TimeToSampleBox::add_sample_delta(unsigned_int32 new_sample_delta)
{
    if(sample_delta.getSize() > 0 && sample_delta[sample_delta.getSize()-1] == new_sample_delta)
        sample_count[sample_count.getSize()-1]++;
    else
    {
        sample_count[sample_count.getSize()] = 1;
        sample_delta[sample_delta.getSize()] = new_sample_delta;
    }

    entry_count = sample_delta.getSize();

    return entry_count;
}

UInt32 TimeToSampleBox::decode_sample_delta()
{
    min_sample_delta = (UInt32)-1;
    max_sample_delta = 0;

    UInt32 sample_delta_count = sample_delta.getSize();

    if(sample_delta.getSize() != sample_count.getSize())
        throw "TimeToSampleBox::decode_sample_delta: Improper sample_delta!";

    for(UInt32 index = 0 ; index < sample_delta_count ; index++)
    {
        for(UInt32 index2 = 0 ; index2 < sample_count[index] ; index2++)
        {
            decoded_sample_delta[decoded_sample_delta.getSize()] = sample_delta[index];
        }

		if(sample_delta[index] > max_sample_delta)
            max_sample_delta = sample_delta[index];

        if(sample_delta[index] < min_sample_delta)
            min_sample_delta = sample_delta[index];
    }

    return decoded_sample_delta.getSize();
}

UInt32 TimeToSampleBox::getCumulativeSampleDelta()
{
    UInt32 cumulativeSampleDelta = 0;
    UInt32 sample_delta_count = sample_delta.getSize();

    if(sample_delta.getSize() != sample_count.getSize())
        throw "TimeToSampleBox::getTotalSampleDuration: Improper sample_delta!";

    for(UInt32 index = 0 ; index < sample_delta_count ; index++)
    {
        cumulativeSampleDelta += (sample_delta[index]*sample_count[index]);
    }

    return cumulativeSampleDelta;
}

void TimeToSampleBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "TimeToSampleBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->writeUInt32(sample_count[index]);
        bufManager->writeUInt32(sample_delta[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 TimeToSampleBox::estimateSize()
{
    return FullBox::estimateSize() + (1+2*entry_count)*sizeof(unsigned_int32);
}

CompositionOffsetBox::CompositionOffsetBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_CompositionOffsetBox, version, 0)
{
    if(version != 0)
        throw "CompositionOffsetBox: Only version 0 supported!";    

    min_sample_offset = (UInt32)-1;
    max_sample_offset = 0;
    entry_count = 0;
}

CompositionOffsetBox::~CompositionOffsetBox()
{
}

void CompositionOffsetBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "CompositionOffsetBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->readUInt32(&sample_count[index]);
        bufManager->readUInt32(&sample_offset[index]);
    }
}

UInt32 CompositionOffsetBox::add_sample_offset(unsigned_int32 new_sample_offset)
{
    if(sample_offset.getSize() > 0 && sample_offset[sample_offset.getSize()-1] == new_sample_offset)
        sample_count[sample_count.getSize()-1]++;
    else
    {
        sample_count[sample_count.getSize()] = 1;
        sample_offset[sample_offset.getSize()] = new_sample_offset;
    }

    entry_count = sample_offset.getSize();

    return entry_count;
}

UInt32 CompositionOffsetBox::decode_sample_offset()
{
    min_sample_offset = (UInt32)-1;
    max_sample_offset = 0;

    UInt32 sample_offset_count = sample_offset.getSize();

    if(sample_offset.getSize() != sample_count.getSize())
        throw "CompositionOffsetBox::decode_sample_offset: Improper sample_offset!";

    for(UInt32 index = 0 ; index < sample_offset_count ; index++)
    {
        for(UInt32 index2 = 0 ; index2 < sample_count[index] ; index2++)
        {
            decoded_sample_offset[decoded_sample_offset.getSize()] = sample_offset[index];
        }
        
        if(sample_offset[index] > max_sample_offset)
            max_sample_offset = sample_offset[index];

        if(sample_offset[index] < min_sample_offset)
            min_sample_offset = sample_offset[index];
    }

    return decoded_sample_offset.getSize();
}

void CompositionOffsetBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "CompositionOffsetBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->writeUInt32(sample_count[index]);
        bufManager->writeUInt32(sample_offset[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 CompositionOffsetBox::estimateSize()
{
    return FullBox::estimateSize() + (1+2*entry_count)*sizeof(unsigned_int32);
}

SampleEntry::SampleEntry(unsigned_int32 format):Box(format)
{
}

SampleEntry::~SampleEntry()
{
}

const unsigned_int8 VisualSampleEntry::reserved_arr[6] = {0,0,0,0,0,0};
const unsigned_int16 VisualSampleEntry::reserved = 0;
const unsigned_int32 VisualSampleEntry::reserved_2 = 0;

VisualSampleEntry::VisualSampleEntry(unsigned_int32 codingname): SampleEntry(codingname)
{
        pre_defined = 0;
        
        memset(pre_defined_arr,0,3*sizeof(unsigned_int32));

        horizresolution = 0x00480000; // 72 dpi
        vertresolution = 0x00480000; // 72 dpi
        frame_count = 1;
        compressorname = new char[32];
        memset(compressorname,0,32*sizeof(char));
        depth = 0x0018;
        pre_defined_2 = -1;

}

VisualSampleEntry::~VisualSampleEntry()
{
    delete [] compressorname;
}

void VisualSampleEntry::set_compressorname(string name)
{
    UInt32 length = (UInt32)strlen(name)+1;

    if(length > 31)
        throw "VisualSampleEntry::set_compressorname: Too long a name!";

    compressorname[0] = length;

    memcpy(&compressorname[1],name,length);
}

void VisualSampleEntry::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "VisualSampleEntry::decode: NULL pointer exception";

	SampleEntry::decode(buffer,count);
        
    UInt32 tempUInt32;
    UInt16 tempUInt16;
    Byte tempByte;

    for(UInt32 index = 0 ; index < 6 ; index++)
    {
        bufManager->readByte(&tempByte);

        if(tempByte != reserved_arr[index])
            throw "VisualSampleEntry::decode: Unexpected reserved value!";
    }

    bufManager->readUInt16(&data_reference_index);

    bufManager->readUInt16(&tempUInt16);
    if(tempUInt16 != pre_defined)
        throw "VisualSampleEntry::decode: Unexpected pre_defined value!";

    bufManager->readUInt16(&tempUInt16);
    if(tempUInt16 != reserved)
        throw "VisualSampleEntry::decode: Unexpected reserved value!";
    
    for(UInt32 index = 0 ; index < 3 ; index++)
    {
        bufManager->readUInt32(&tempUInt32);

        if(tempUInt32 != pre_defined_arr[index])
            throw "VisualSampleEntry::decode: Unexpected pre_defined value!";
    }

    bufManager->readUInt16(&width);
    bufManager->readUInt16(&height);

    bufManager->readUInt32(&tempUInt32);
    if(tempUInt32 != horizresolution)
        throw "VisualSampleEntry::decode: Unexpected horizresolution value!";

    bufManager->readUInt32(&tempUInt32);
    if(tempUInt32 != vertresolution)
        throw "VisualSampleEntry::decode: Unexpected vertresolution value!";

    bufManager->readUInt32(&tempUInt32);
    if(tempUInt32 != reserved_2)
        throw "VisualSampleEntry::decode: Unexpected reserved value!";

    bufManager->readUInt16(&tempUInt16);
    if(tempUInt16 != frame_count)
        throw "VisualSampleEntry::decode: Unexpected frame_count value!";

    bufManager->readData((Byte *)compressorname, 32);

    bufManager->readUInt16(&tempUInt16);
    if(tempUInt16 != depth)
        throw "VisualSampleEntry::decode: Unexpected depth value!";

    bufManager->readUInt16(&tempUInt16);
    if((int16)tempUInt16 != pre_defined_2)
        throw "VisualSampleEntry::decode: Unexpected pre_defined value!";
}

void VisualSampleEntry::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "VisualSampleEntry::encode: NULL pointer exception";

    SampleEntry::encode(buffer,count);
   
    for(UInt32 index = 0 ; index < 6 ; index++)
    {
        bufManager->writeByte(reserved_arr[index]);
    }

    bufManager->writeUInt16(data_reference_index);
    bufManager->writeUInt16(pre_defined);
    bufManager->writeUInt16(reserved);
    
    for(UInt32 index = 0 ; index < 3 ; index++)
    {
        bufManager->writeUInt32(pre_defined_arr[index]);
    }

    bufManager->writeUInt16(width);
    bufManager->writeUInt16(height);
    bufManager->writeUInt32(horizresolution);
    bufManager->writeUInt32(vertresolution);
    bufManager->writeUInt32(reserved_2);
    bufManager->writeUInt16(frame_count);
    bufManager->writeData((Byte *)compressorname, 32);
    bufManager->writeUInt16(depth);
    bufManager->writeUInt16(pre_defined_2);

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 VisualSampleEntry::estimateSize()
{
    return SampleEntry::estimateSize() + 6*sizeof(unsigned_int8) + 3*sizeof(unsigned_int16) + 3*sizeof(unsigned_int32) + 2*sizeof(unsigned_int16)
            + 3*sizeof(unsigned_int32) + sizeof(unsigned_int16) + 32*sizeof(Byte) + 2*sizeof(unsigned_int16);
}

SampleDescriptionBox::SampleDescriptionBox(UInt32 handler_type_index):FullBox(IsoBoxTypes::type_SampleDescriptionBox, 0, 0)
{
    mapStringToUInt32("soun", &handler_types[0]);
    mapStringToUInt32("vide", &handler_types[1]);
    mapStringToUInt32("hint", &handler_types[2]);

    entry_count = 0;
    
    setHandlerType(handler_type_index);
}

void SampleDescriptionBox::setHandlerType(UInt32 handler_type_index)
{
    if(handler_type_index != 1)
        throw "SampleDescriptionBox: Only vide VisualSampleEntry supported!"; 

    memcpy(&(this->handler_type),&handler_types[handler_type_index],sizeof(unsigned_int32));
}

SampleDescriptionBox::~SampleDescriptionBox()
{
    for(UInt32 index = 0 ; index < sampleEntry.getSize() ; index++)
        delete sampleEntry[index];
}

SampleEntry * SampleDescriptionBox::addNewSampleEntry(unsigned_int32 handler_type)
{
    if(handler_type == handler_types[1])
        sampleEntry[sampleEntry.getSize()] = new VisualSampleEntry(0);
    else
        throw "SampleDescriptionBox:addNewSampleEntry: Only vide VisualSampleEntry supported!";

    entry_count = sampleEntry.getSize();

    return sampleEntry[sampleEntry.getSize()-1];
}

void SampleDescriptionBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "SampleDescriptionBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        UInt32 initialReadPointer = getCurrentReadPointer();
        
        if(handler_type == handler_types[1])
        {
            addNewSampleEntry(handler_type);
            sampleEntry[index]->decode(&(buffer[initialReadPointer]),count-initialReadPointer);
        }
        else
            throw "SampleDescriptionBox:decode: Only vide VisualSampleEntry supported!";

        bufManager->setReadPointer(initialReadPointer+sampleEntry[index]->size);
    }
}

void SampleDescriptionBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "CompositionOffsetBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    entry_count = sampleEntry.getSize();
    
    bufManager->writeUInt32(entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        if(handler_type == handler_types[1])
        {
            Byte *sampleBuffer;
            UInt32 SampleCount;
            sampleEntry[index]->encode(&sampleBuffer,&SampleCount);
            bufManager->writeData(sampleBuffer,SampleCount);
        }
        else
            throw "SampleDescriptionBox:encode: Only vide VisualSampleEntry supported!"; 
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 SampleDescriptionBox::estimateSize()
{
    UInt32 totalSize = FullBox::estimateSize() + sizeof(unsigned_int32);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        if(handler_type == handler_types[1])
        {
            totalSize += sampleEntry[index]->estimateSize();
        }
        else
            throw "SampleDescriptionBox:estimateSize: Only vide VisualSampleEntry supported!"; 
    }
    
    return totalSize;
}

SampleSizeBox::SampleSizeBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_SampleSizeBox, version, 0)
{
    if(version != 0)
        throw "SampleSizeBox: Only version 0 supported!";

    sample_count = 0;
    sample_size = 0;
}

SampleSizeBox::~SampleSizeBox()
{
}

void SampleSizeBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "SampleSizeBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&sample_size);
    bufManager->readUInt32(&sample_count);

    if(sample_size == 0)
        for(UInt32 index = 0 ; index < sample_count ; index++)
        {
            bufManager->readUInt32(&entry_size[index]);
        }
}

UInt32 SampleSizeBox::add_sample_size(unsigned_int32 new_sample_size)
{
    if(sample_count > 0)
    {
        if(sample_size != 0 && new_sample_size != sample_size)  //If all previous samples were equal in size, but this one is not, init entry_size table
        {
            for(UInt32 index = 0 ; index < sample_count; index++)
            {
                entry_size[entry_size.getSize()]= sample_size;
            }
            sample_size = 0;    //From now on, all entries in table
        }
    }
    
    if(sample_size == 0 && sample_count > 0)    //If we had to initialize the table previously, add the new entry in entry_size table
        entry_size[entry_size.getSize()] = new_sample_size;

    if(entry_size.getSize() == 0)   //true if all previous samples were of equal size (since table not initialized) OR first sample size
        sample_size = new_sample_size;

    return sample_size;
}

void SampleSizeBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "SampleSizeBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(sample_size);
    bufManager->writeUInt32(sample_count);

    if(sample_size == 0)
        for(UInt32 index = 0 ; index < sample_count ; index++)
        {
            bufManager->writeUInt32(entry_size[index]);
        }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 SampleSizeBox::estimateSize()
{
    return FullBox::estimateSize() + ( 2 + (sample_size == 0 ? sample_count : 0))*sizeof(unsigned_int32);
}

SampleToChunkBox::SampleToChunkBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_SampleToChunkBox, version, 0)
{
    if(version != 0)
        throw "SampleSizeBox: Only version 0 supported!";

    entry_count = 0;
}

SampleToChunkBox::~SampleToChunkBox()
{
}

void SampleToChunkBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "SampleToChunkBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->readUInt32(&first_chunk[index]);
        bufManager->readUInt32(&samples_per_chunk[index]);
        bufManager->readUInt32(&sample_description_index[index]);
    }
}

UInt32 SampleToChunkBox::add_entry(unsigned_int32 first_chunk,unsigned_int32 samples_per_chunk,unsigned_int32 sample_description_index)
{
    this->first_chunk[entry_count] = first_chunk;
    this->samples_per_chunk[entry_count] = samples_per_chunk;
    this->sample_description_index[entry_count] = sample_description_index;
    return ++entry_count;
}

void SampleToChunkBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "SampleToChunkBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->writeUInt32(first_chunk[index]);
        bufManager->writeUInt32(samples_per_chunk[index]);
        bufManager->writeUInt32(sample_description_index[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 SampleToChunkBox::estimateSize()
{
    return FullBox::estimateSize() + (1+3*entry_count)*sizeof(unsigned_int32);
}

ChunkOffsetBox::ChunkOffsetBox(unsigned_int8 version):FullBox(IsoBoxTypes::type_ChunkOffsetBox, version, 0)
{
    if(version != 0)
        throw "ChunkOffsetBox: Only version 0 supported!";

    entry_count = 0;
}

ChunkOffsetBox::~ChunkOffsetBox()
{
}

void ChunkOffsetBox::decode(Byte *buffer, UInt32 count)
{
    if(buffer == NULL)
        throw "ChunkOffsetBox::decode: NULL pointer exception";

    FullBox::decode(buffer,count);

    bufManager->readUInt32(&entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->readUInt32(&chunk_offset[index]);
    }
}

UInt32 ChunkOffsetBox::add_chunk_offset(unsigned_int32 chunk_offset)
{
    this->chunk_offset[entry_count] = chunk_offset;

    return ++entry_count;
}

void ChunkOffsetBox::encode(Byte **buffer, UInt32 *count)
{
    if(buffer == NULL || count == NULL)
        throw "ChunkOffsetBox::encode: NULL pointer exception";

    FullBox::encode(buffer,count);
   
    bufManager->writeUInt32(entry_count);

    for(UInt32 index = 0 ; index < entry_count ; index++)
    {
        bufManager->writeUInt32(chunk_offset[index]);
    }

    update_size();
    
    *count = getBuffer(buffer);
}

UInt32 ChunkOffsetBox::estimateSize()
{
    return FullBox::estimateSize() + (1+entry_count)*sizeof(unsigned_int32);
}

